@@ -15,7 +15,7 @@ package MCE::Core::Input::Generator;
 use strict;
 use warnings;
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## Items below are folded into MCE.
 
@@ -73,7 +73,7 @@ sub _worker_sequence_generator {
       if ($_wid == 1) {
          $self->{_next_jmp} = sub { goto _WORKER_SEQ_GEN__LAST; };
 
-         $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
+         $_ = (defined $_fmt) ? _sprintf("%$_fmt", $_next) : $_next;
 
          if ($_chunk_size > 1 || $_bounds_only) {
             $_ = ($_bounds_only) ? [ $_, $_ ] : [ $_ ];
@@ -92,7 +92,7 @@ sub _worker_sequence_generator {
          return if ( $_flag && $_next > $_end);
          return if (!$_flag && $_next < $_end);
 
-         $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
+         $_ = (defined $_fmt) ? _sprintf("%$_fmt", $_next) : $_next;
          $_ = [ $_, $_ ] if ($_bounds_only);
 
          $_wuf->($self, $_, $_chunk_id);
@@ -115,10 +115,13 @@ sub _worker_sequence_generator {
          if ($_bounds_only) {
             my ($_tmp_b, $_tmp_e) = ($_next);
 
-            if ($_begin < $_end) {
+            if ($_begin <= $_end) {
                if ($_step * ($_chunk_size - 1) + $_n_begin <= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
+               elsif ($_step == 1) {
+                  $_tmp_e = $_end if ($_next <= $_end);
+               }
                else {
                   for my $_i (1 .. $_chunk_size) {
                      last if ($_next > $_end);
@@ -131,6 +134,9 @@ sub _worker_sequence_generator {
                if ($_step * ($_chunk_size - 1) + $_n_begin >= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
+               elsif ($_step == -1) {
+                  $_tmp_e = $_end if ($_next >= $_end);
+               }
                else {
                   for my $_i (1 .. $_chunk_size) {
                      last if ($_next < $_end);
@@ -143,15 +149,15 @@ sub _worker_sequence_generator {
             return unless (defined $_tmp_e);
 
             @_n = (defined $_fmt)
-               ? ( sprintf("%$_fmt",$_tmp_b), sprintf("%$_fmt",$_tmp_e) )
+               ? ( _sprintf("%$_fmt",$_tmp_b), _sprintf("%$_fmt",$_tmp_e) )
                : ( $_tmp_b, $_tmp_e );
          }
 
          ## -------------------------------------------------------------------
 
          else {
-            if ($_begin < $_end) {
-               if (!defined $_fmt && $_step == 1) {
+            if ($_begin <= $_end) {
+               if (!defined $_fmt && $_step == 1 && abs($_end) < ~1 && abs($_begin) < ~1) {
                   @_n = ($_next + $_chunk_size <= $_end)
                      ? ($_next .. $_next + $_chunk_size - 1)
                      : ($_next .. $_end);
@@ -161,7 +167,7 @@ sub _worker_sequence_generator {
                      last if ($_next > $_end);
 
                      push @_n, (defined $_fmt)
-                        ? sprintf("%$_fmt", $_next) : $_next;
+                        ? _sprintf("%$_fmt", $_next) : $_next;
 
                      $_next = $_step * $_i + $_n_begin;
                   }
@@ -172,7 +178,7 @@ sub _worker_sequence_generator {
                   last if ($_next < $_end);
 
                   push @_n, (defined $_fmt)
-                     ? sprintf("%$_fmt", $_next) : $_next;
+                     ? _sprintf("%$_fmt", $_next) : $_next;
 
                   $_next = $_step * $_i + $_n_begin;
                }
@@ -200,3 +206,37 @@ sub _worker_sequence_generator {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Input::Generator - Sequence of numbers (for task_id > 0)
+
+=head1 VERSION
+
+This document describes MCE::Core::Input::Generator version 1.867_01
+
+=head1 DESCRIPTION
+
+This package provides a sequence of numbers used internally by the worker
+process. Distribution is divided equally among workers. This allows sequence
+to be configured independently among multiple user tasks.
+
+There is no public API.
+
+=head1 SEE ALSO
+
+The syntax for the C<sequence> option is described in L<MCE::Core>.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
