@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## File_path and Scalar_ref input reader.
+## File path and Scalar reference input reader.
 ##
 ## This package provides the read handle method used internally by the worker
 ## process. Distribution follows a bank-queuing model.
@@ -14,7 +14,7 @@ package MCE::Core::Input::Handle;
 use strict;
 use warnings;
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## Items below are folded into MCE.
 
@@ -23,9 +23,6 @@ package # hide from rpm
 
 no warnings qw( threads recursion uninitialized );
 
-use Fcntl qw( SEEK_CUR );
-use bytes;
-
 my $_que_read_size = $MCE::_que_read_size;
 my $_que_template  = $MCE::_que_template;
 
@@ -36,9 +33,10 @@ my $_que_template  = $MCE::_que_template;
 ###############################################################################
 
 sub _systell {
-   my $_off;
-   1 until defined ( $_off = sysseek($_[0], 0, SEEK_CUR) );
-   $_off;
+   # To minimize memory consumption, SEEK_CUR equals 1 on most platforms.
+   # e.g. use Fcntl qw(SEEK_CUR);
+
+   sysseek($_[0], 0, 1);
 }
 
 sub _worker_read_handle {
@@ -50,17 +48,35 @@ sub _worker_read_handle {
    _croak('MCE::_worker_read_handle: (user_func) is not specified')
       unless (defined $self->{user_func});
 
+   my $_DAT_LOCK    = $self->{_dat_lock};
    my $_QUE_R_SOCK  = $self->{_que_r_sock};
    my $_QUE_W_SOCK  = $self->{_que_w_sock};
+   my $_lock_chn    = $self->{_lock_chn};
    my $_chunk_size  = $self->{chunk_size};
    my $_use_slurpio = $self->{use_slurpio};
    my $_parallel_io = $self->{parallel_io};
    my $_RS          = $self->{RS} || $/;
-   my $_RS_FLG      = (!$_RS || $_RS ne $LF);
    my $_wuf         = $self->{_wuf};
 
    my ($_data_size, $_next, $_chunk_id, $_offset_pos, $_IN_FILE, $_tmp_cs);
-   my ($_chop_len, $_chop_str, $_p);
+   my ($_dat_ex, $_dat_un, $_pid, $_chop_len, $_chop_str, $_p);
+
+   if ($_lock_chn) {
+      $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
+
+      # inlined for performance
+      if ($self->{_data_channels} > 5) {
+         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 5 + 1 )};
+      }
+      $_dat_ex = sub {
+         MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+            unless $_DAT_LOCK->{ $_pid };
+      };
+      $_dat_un = sub {
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            if $_DAT_LOCK->{ $_pid };
+      };
+   }
 
    if (length $_RS > 1 && substr($_RS, 0, 1) eq "\n") {
       $_chop_str = substr($_RS, 1);
@@ -88,16 +104,19 @@ sub _worker_read_handle {
    _WORKER_READ_HANDLE__NEXT:
 
    while (1) {
-      my @_recs; undef $_ if (length > MAX_CHUNK_SIZE);
+      my @_recs; undef $_ if (length > MAX_GC_SIZE);
 
       $_ = '';
 
       ## Obtain the next chunk_id and offset position.
-      1 until sysread $_QUE_R_SOCK, $_next, $_que_read_size;
+      $_dat_ex->() if $_lock_chn;
+      MCE::Util::_sysread($_QUE_R_SOCK, $_next, $_que_read_size);
+
       ($_chunk_id, $_offset_pos) = unpack($_que_template, $_next);
 
       if ($_offset_pos >= $_data_size) {
-         1 until syswrite $_QUE_W_SOCK, pack($_que_template, 0, $_offset_pos);
+         syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset_pos));
+         $_dat_un->() if $_lock_chn;
          close $_IN_FILE; undef $_IN_FILE;
          return;
       }
@@ -110,7 +129,7 @@ sub _worker_read_handle {
 
       ## Read data.
       if ($_chunk_size <= MAX_RECS_SIZE) {        # One or many records.
-         local $/ = $_RS if ($_RS_FLG);
+         local $/ = $_RS if ($/ ne $_RS);
          seek $_IN_FILE, $_offset_pos, 0;
 
          if ($_chunk_size == 1) {
@@ -151,15 +170,20 @@ sub _worker_read_handle {
             }
          }
 
-         1 until syswrite $_QUE_W_SOCK,
-            pack($_que_template, $_chunk_id, tell $_IN_FILE);
+         syswrite(
+            $_QUE_W_SOCK, pack($_que_template, $_chunk_id, tell $_IN_FILE)
+         );
+         $_dat_un->() if $_lock_chn;
       }
       else {                                      # Large chunk.
-         local $/ = $_RS if ($_RS_FLG);
+         local $/ = $_RS if ($/ ne $_RS);
 
-         if ($_parallel_io && ! $_RS_FLG) {
-            1 until syswrite $_QUE_W_SOCK,
-               pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size);
+         if ($_parallel_io && $_RS eq $LF) {
+            syswrite(
+               $_QUE_W_SOCK,
+               pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size)
+            );
+            $_dat_un->() if $_lock_chn;
 
             $_tmp_cs = $_chunk_size;
             seek $_IN_FILE, $_offset_pos, 0;
@@ -169,9 +193,9 @@ sub _worker_read_handle {
             }
 
             if ($_proc_type == READ_FILE) {
-               1 until defined ( sysseek $_IN_FILE, tell( $_IN_FILE ), 0 );
-               1 until defined ( sysread $_IN_FILE, $_, $_tmp_cs, $_p );
-               seek $_IN_FILE, _systell( $_IN_FILE ), 0;
+               sysseek($_IN_FILE, tell( $_IN_FILE ), 0);
+               sysread($_IN_FILE, $_, $_tmp_cs, $_p);
+               seek $_IN_FILE, _systell($_IN_FILE), 0;
             }
             else {
                read $_IN_FILE, $_, $_tmp_cs, $_p;
@@ -181,9 +205,9 @@ sub _worker_read_handle {
          }
          else {
             if ($_proc_type == READ_FILE) {
-               1 until defined ( sysseek $_IN_FILE, $_offset_pos, 0 );
-               1 until defined ( sysread $_IN_FILE, $_, $_chunk_size, $_p );
-               seek $_IN_FILE, _systell( $_IN_FILE ), 0;
+               sysseek($_IN_FILE, $_offset_pos, 0);
+               sysread($_IN_FILE, $_, $_chunk_size, $_p);
+               seek $_IN_FILE, _systell($_IN_FILE), 0;
             }
             else {
                seek $_IN_FILE, $_offset_pos, 0;
@@ -192,8 +216,10 @@ sub _worker_read_handle {
 
             $_ .= <$_IN_FILE>;
 
-            1 until syswrite $_QUE_W_SOCK,
-               pack($_que_template, $_chunk_id, tell $_IN_FILE);
+            syswrite(
+               $_QUE_W_SOCK, pack($_que_template, $_chunk_id, tell $_IN_FILE)
+            );
+            $_dat_un->() if $_lock_chn;
          }
       }
 
@@ -214,7 +240,7 @@ sub _worker_read_handle {
          }
          else {
             if ($_chunk_size > MAX_RECS_SIZE) {
-               local $/ = $_RS if ($_RS_FLG);
+               local $/ = $_RS if ($/ ne $_RS);
                _sync_buffer_to_array(\$_, \@_recs, $_chop_str);
                undef $_;
             }
@@ -240,3 +266,36 @@ sub _worker_read_handle {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Input::Handle - File path and Scalar reference input reader
+
+=head1 VERSION
+
+This document describes MCE::Core::Input::Handle version 1.867_01
+
+=head1 DESCRIPTION
+
+This package provides the read handle method used internally by the worker
+process. Distribution follows a bank-queuing model.
+
+There is no public API.
+
+=head1 SEE ALSO
+
+The syntax for the C<input_data> option is described in L<MCE::Core>.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
