@@ -14,7 +14,7 @@ package MCE::Core::Input::Iterator;
 use strict;
 use warnings;
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## Items below are folded into MCE.
 
@@ -23,14 +23,14 @@ package # hide from rpm
 
 no warnings qw( threads recursion uninitialized );
 
-use bytes;
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Worker process -- User Iterator.
 ##
 ###############################################################################
 
+my $_is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
+
 sub _worker_user_iterator {
 
    my ($self) = @_;
@@ -46,17 +46,25 @@ sub _worker_user_iterator {
    my $_DAU_W_SOCK  = $self->{_dat_w_sock}->[$_chn];
    my $_lock_chn    = $self->{_lock_chn};
    my $_chunk_size  = $self->{chunk_size};
-   my $_I_FLG       = (!$/ || $/ ne $LF);
    my $_wuf         = $self->{_wuf};
 
-   my ($_dat_ex, $_dat_un);
+   my ($_dat_ex, $_dat_un, $_pid);
 
    if ($_lock_chn) {
-      $_dat_ex = sub { 1 until sysread(  $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
-      $_dat_un = sub { 1 until syswrite( $_DAT_LOCK->{_w_sock}, '0' ) };
+      $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
+
+      # inlined for performance
+      $_dat_ex = sub {
+         MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+            unless $_DAT_LOCK->{ $_pid };
+      };
+      $_dat_un = sub {
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            if $_DAT_LOCK->{ $_pid };
+      };
    }
 
-   my ($_chunk_id, $_len, $_is_ref);
+   my ($_chunk_id, $_len);
 
    ## -------------------------------------------------------------------------
 
@@ -68,16 +76,18 @@ sub _worker_user_iterator {
    _WORKER_USER_ITERATOR__NEXT:
 
    while (1) {
-      undef $_ if (length > MAX_CHUNK_SIZE);
+      undef $_ if (length > MAX_GC_SIZE);
 
       $_ = '';
 
       ## Obtain the next chunk of data.
       {
-         local $\ = undef if (defined $\); local $/ = $LF if ($_I_FLG);
+         local $\ = undef if (defined $\);
+         local $/ = $LF   if ($/ ne $LF );
 
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_U_ITR . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_I_REF . $LF . $_chn . $LF;
+         MCE::Util::_sock_ready($_DAU_W_SOCK, -1) if $_is_MSWin32;
          chomp($_len = <$_DAU_W_SOCK>);
 
          if ($_len < 0) {
@@ -85,8 +95,6 @@ sub _worker_user_iterator {
             return;
          }
 
-         $_is_ref = chop $_len;
-
          chomp($_chunk_id = <$_DAU_W_SOCK>);
          read $_DAU_W_SOCK, $_, $_len;
 
@@ -94,14 +102,9 @@ sub _worker_user_iterator {
       }
 
       ## Call user function.
-      if ($_is_ref) {
-         my $_chunk_ref = $self->{thaw}($_); undef $_;
-         $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
-         $_wuf->($self, $_chunk_ref, $_chunk_id);
-      }
-      else {
-         $_wuf->($self, [ $_ ], $_chunk_id);
-      }
+      my $_chunk_ref = $self->{thaw}($_); undef $_;
+      $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
+      $_wuf->($self, $_chunk_ref, $_chunk_id);
    }
 
    _WORKER_USER_ITERATOR__LAST:
@@ -111,3 +114,36 @@ sub _worker_user_iterator {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Input::Iterator - Iterator reader
+
+=head1 VERSION
+
+This document describes MCE::Core::Input::Iterator version 1.867_01
+
+=head1 DESCRIPTION
+
+This package, used interally by the worker process, provides support for
+user specified iterators assigned to C<input_data>.
+
+There is no public API.
+
+=head1 SEE ALSO
+
+The syntax for the C<input_data> option is described in L<MCE::Core>.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
