@@ -14,7 +14,7 @@ package MCE::Core::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -26,7 +26,13 @@ package # hide from rpm
 
 no warnings qw( threads recursion uninitialized );
 
-use bytes;
+## The POSIX module has many symbols. Try not loading it simply
+## to have WNOHANG. The following covers most platforms.
+
+use constant {
+   _WNOHANG => ( $INC{'POSIX.pm'} )
+      ? &POSIX::WNOHANG : ( $^O eq 'solaris' ) ? 64 : 1
+};
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -53,6 +59,9 @@ sub _task_end {
          $_task_end->($self, $_task_id, $_task_name);
       }
    }
+   elsif (defined $self->{task_end}) {
+      $self->{task_end}->($self, 0, $self->{task_name});
+   }
 
    return;
 }
@@ -66,6 +75,35 @@ sub _task_end {
 ##
 ###############################################################################
 
+my %_sendto_fhs;
+
+sub _sendto_fhs_close {
+   for my $_p (keys %_sendto_fhs) {
+      close  $_sendto_fhs{$_p};
+      delete $_sendto_fhs{$_p};
+   }
+}
+
+sub _sendto_fhs_get {
+   my ($self, $_fd) = @_;
+
+   $_sendto_fhs{$_fd} || do {
+
+      $_sendto_fhs{$_fd} = IO::Handle->new();
+      $_sendto_fhs{$_fd}->fdopen($_fd, 'w')
+         or _croak "Cannot open file descriptor ($_fd): $!";
+
+      binmode $_sendto_fhs{$_fd};
+
+      if (!exists $self->{flush_file} || $self->{flush_file}) {
+         local $!;
+         $_sendto_fhs{$_fd}->autoflush(1)
+      }
+
+      $_sendto_fhs{$_fd};
+   };
+}
+
 sub _output_loop {
 
    my ( $self, $_input_data, $_input_glob, $_plugin_function,
@@ -74,47 +112,41 @@ sub _output_loop {
    @_ = ();
 
    my (
-      $_aborted, $_eof_flag, $_max_retries, $_syn_flag, %_sendto_fhs,
-      $_cb, $_chunk_id, $_chunk_size, $_fd, $_file, $_flush_file, $_wa,
+      $_aborted, $_eof_flag, $_max_retries, $_syn_flag, $_win32_ipc,
+      $_cb, $_chunk_id, $_chunk_size, $_file, $_size_completed, $_wa,
       @_is_c_ref, @_is_h_ref, @_is_q_ref, $_on_post_exit, $_on_post_run,
       $_has_user_tasks, $_sess_dir, $_task_id, $_user_error, $_user_output,
       $_input_size, $_offset_pos, $_single_dim, @_gather, $_cs_one_flag,
       $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
-      $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
-      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT
+      $_BSB_W_SOCK, $_BSB_R_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
+      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT,
+      @_delay_wid
    );
 
    ## -------------------------------------------------------------------------
    ## Callback return.
 
-   my $_cb_ret_a = sub {                          # CBK return array
+   my $_cb_reply = sub {
+      local $\ = $_O_SEP if ($_O_FLG);
+      local $/ = $_I_SEP if ($_I_FLG);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
+      no strict 'refs';
 
-      print {$_DAU_R_SOCK} $_len.$LF, $_buf;
+      if ( $_wa == WANTS_UNDEF ) {
+         $_cb->(@_);
+         return;
+      }
+      elsif ( $_wa == WANTS_ARRAY ) {
+         my @_ret = $_cb->(@_);
+         my $_buf = $self->{freeze}(\@_ret);
 
-      return;
-   };
+         return print {$_DAU_R_SOCK} length($_buf).$LF, $_buf;
+      }
 
-   my $_cb_ret_r = sub {                          # CBK return reference
+      my $_ret = $_cb->(@_);
+      my $_buf = $self->{freeze}([ $_ret ]);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
-
-      print {$_DAU_R_SOCK} WANTS_REF.$LF . $_len.$LF, $_buf;
-
-      return;
-   };
-
-   my $_cb_ret_s = sub {                          # CBK return scalar
-
-      $_len = (defined $_[0]) ? length $_[0] : -1;
-      local $\ = undef if (defined $\);
-
-      print {$_DAU_R_SOCK} WANTS_SCALAR.$LF . $_len.$LF, $_[0];
-
-      return;
+      return print {$_DAU_R_SOCK} length($_buf).$LF, $_buf;
    };
 
    ## -------------------------------------------------------------------------
@@ -141,12 +173,14 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  syswrite($_BSB_W_SOCK, $LF);
+               }
                undef $_syn_flag;
             }
          }
 
-         _task_end($self, $_task_id) unless ($_total_running);
+         _task_end($self, $_task_id) unless $_total_running;
 
          return;
       },
@@ -171,7 +205,9 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  syswrite($_BSB_W_SOCK, $LF);
+               }
                undef $_syn_flag;
             }
          }
@@ -194,10 +230,10 @@ sub _output_loop {
             $self->{_wrk_status} = $_exit_status;
          }
 
-         print {$_DAU_R_SOCK} $LF unless ($^O eq 'MSWin32');
-
          ## Reap child/thread. Note: Win32 uses negative PIDs.
 
+         local ($!, $?);
+
          if ($_exit_pid =~ /^PID_(-?\d+)/) {
             my $_pid = $1; my $_list = $self->{_pids};
             for my $i (0 .. @{ $_list }) {
@@ -212,7 +248,7 @@ sub _output_loop {
             my $_tid = $1; my $_list = $self->{_tids};
             for my $i (0 .. @{ $_list }) {
                if ($_list->[$i] && $_list->[$i] == $_tid) {
-                  ${ $self->{_thrs}->[$i] }->join();
+                  eval { $self->{_thrs}->[$i]->join() };
                   $self->{_thrs}->[$i] = undef;
                   $self->{_tids}->[$i] = undef;
                   last;
@@ -227,13 +263,13 @@ sub _output_loop {
             $self->{_exited_wid} = $_exit_wid;
 
             if (length($_retry_buf)) {
-               $self->{_retry} = $self->{thaw}($_retry_buf);
-               my $_retry_cnt  = $_max_retries - $self->{_retry}[2] - 1;
+               $self->{_retry}     = $self->{thaw}($_retry_buf);
+               $self->{_retry_cnt} = $_max_retries - $self->{_retry}[2] - 1;
 
                $_on_post_exit->($self, {
                   wid => $_exit_wid, pid => $_exit_pid, status => $_exit_status,
                   msg => $_exit_msg, id  => $_exit_id
-               }, $_retry_cnt);
+               }, $self->{_retry_cnt});
 
                delete $self->{_retry};
             }
@@ -253,14 +289,14 @@ sub _output_loop {
             };
          }
 
-         _task_end($self, $_task_id) unless ($_total_running);
+         _task_end($self, $_task_id) unless $_total_running;
 
          return;
       },
 
       ## ----------------------------------------------------------------------
 
-      OUTPUT_A_ARY.$LF => sub {                   # Array << Array
+      OUTPUT_A_REF.$LF => sub {                   # Input << Array ref
          my $_buf;
 
          if ($_offset_pos >= $_input_size || $_aborted) {
@@ -271,7 +307,7 @@ sub _output_loop {
          }
 
          if ($_single_dim && $_cs_one_flag) {
-            $_buf = $_input_data->[$_offset_pos];
+            $_buf = $self->{freeze}( [ $_input_data->[$_offset_pos] ] );
          }
          else {
             if ($_offset_pos + $_chunk_size - 1 < $_input_size) {
@@ -293,7 +329,7 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_S_GLB.$LF => sub {                   # Scalar << Glob FH
+      OUTPUT_G_REF.$LF => sub {                   # Input << Glob ref
          my $_buf = '';
 
          ## The logic below honors ('Ctrl/Z' in Windows, 'Ctrl/D' in Unix)
@@ -312,13 +348,15 @@ sub _output_loop {
 
             if ($_chunk_size <= MAX_RECS_SIZE) {
                if ($_chunk_size == 1) {
-                  $_buf = <$_input_glob>;
+                  $_buf .= $_input_glob->can('getline')
+                     ? $_input_glob->getline : <$_input_glob>;
                   $_eof_flag = 1 unless (length $_buf);
                }
                else {
                   my $_last_len = 0;
                   for (1 .. $_chunk_size) {
-                     $_buf .= <$_input_glob>;
+                     $_buf .= $_input_glob->can('getline')
+                        ? $_input_glob->getline : <$_input_glob>;
                      $_len  = length $_buf;
                      if ($_len == $_last_len) {
                         $_eof_flag = 1;
@@ -329,12 +367,21 @@ sub _output_loop {
                }
             }
             else {
-               if (read($_input_glob, $_buf, $_chunk_size) == $_chunk_size) {
-                  $_buf .= <$_input_glob>;
-                  $_eof_flag = 1 if (length $_buf == $_chunk_size);
+               if ($_input_glob->can('getline') && $_input_glob->can('read')) {
+                  if ($_input_glob->read($_buf, $_chunk_size) == $_chunk_size) {
+                     $_buf .= $_input_glob->getline;
+                     $_eof_flag = 1 if (length $_buf == $_chunk_size);
+                  } else {
+                     $_eof_flag = 1;
+                  }
                }
                else {
-                  $_eof_flag = 1;
+                  if (read($_input_glob, $_buf, $_chunk_size) == $_chunk_size) {
+                     $_buf .= <$_input_glob>;
+                     $_eof_flag = 1 if (length $_buf == $_chunk_size);
+                  } else {
+                     $_eof_flag = 1;
+                  }
                }
             }
          }
@@ -342,15 +389,47 @@ sub _output_loop {
          $_len = length $_buf; local $\ = undef if (defined $\);
 
          if ($_len) {
-            print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
-         } else {
+            my $_tmp = $self->{freeze}(\$_buf);
+            print {$_DAU_R_SOCK} length($_tmp).$LF . (++$_chunk_id).$LF, $_tmp;
+         }
+         else {
             print {$_DAU_R_SOCK} '0'.$LF;
          }
 
          return;
       },
 
-      OUTPUT_U_ITR.$LF => sub {                   # User << Iterator
+      OUTPUT_H_REF.$LF => sub {                   # Input << Hash ref
+         my @_pairs;
+
+         if ($_offset_pos >= $_input_size || $_aborted) {
+            local $\ = undef if (defined $\);
+            print {$_DAU_R_SOCK} '0'.$LF;
+
+            return;
+         }
+
+         if ($_offset_pos + $_chunk_size - 1 < $_input_size) {
+            for my $_i ($_offset_pos .. $_offset_pos + $_chunk_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+         else {
+            for my $_i ($_offset_pos .. $_input_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+
+         my $_buf = $self->{freeze}(\@_pairs);
+
+         $_len = length $_buf; local $\ = undef if (defined $\);
+         print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
+         $_offset_pos += $_chunk_size;
+
+         return;
+      },
+
+      OUTPUT_I_REF.$LF => sub {                   # Input << Iter ref
          my $_buf;
 
          if ($_aborted) {
@@ -362,16 +441,10 @@ sub _output_loop {
 
          my @_ret_a = $_input_data->($_chunk_size);
 
-         if (scalar @_ret_a > 1 || ref $_ret_a[0]) {
-            $_buf = $self->{freeze}( [ @_ret_a ] );
+         if (@_ret_a > 1 || defined $_ret_a[0]) {
+            $_buf = $self->{freeze}([ @_ret_a ]);
             $_len = length $_buf; local $\ = undef if (defined $\);
-            print {$_DAU_R_SOCK} $_len.'1'.$LF . (++$_chunk_id).$LF, $_buf;
-
-            return;
-         }
-         elsif (defined $_ret_a[0]) {
-            $_len = length $_ret_a[0]; local $\ = undef if (defined $\);
-            print {$_DAU_R_SOCK} $_len.'0'.$LF . (++$_chunk_id).$LF, $_ret_a[0];
+            print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
 
             return;
          }
@@ -385,93 +458,31 @@ sub _output_loop {
 
       ## ----------------------------------------------------------------------
 
-      OUTPUT_A_CBK.$LF => sub {                   # Callback w/ multiple args
-         my ($_buf, $_data_ref);
-
+      OUTPUT_A_CBK.$LF => sub {                   # Callback w/ args
          chomp($_wa  = <$_DAU_R_SOCK>),
          chomp($_cb  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
-         $_data_ref = $self->{thaw}($_buf); undef $_buf;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
+         my $_aref = $self->{thaw}($_buf);
+         undef $_buf;
 
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->(@{ $_data_ref });
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->(@{ $_data_ref });
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->(@{ $_data_ref });
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
-      },
-
-      OUTPUT_S_CBK.$LF => sub {                   # Callback w/ 1 scalar arg
-         my $_buf;
-
-         chomp($_wa  = <$_DAU_R_SOCK>),
-         chomp($_cb  = <$_DAU_R_SOCK>),
-         chomp($_len = <$_DAU_R_SOCK>);
-
-         read $_DAU_R_SOCK, $_buf, $_len;
-
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->($_buf);
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->($_buf);
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->($_buf);
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->(@{ $_aref });
       },
 
       OUTPUT_N_CBK.$LF => sub {                   # Callback w/ no args
-
          chomp($_wa = <$_DAU_R_SOCK>),
          chomp($_cb = <$_DAU_R_SOCK>);
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->();
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->();
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->();
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->();
       },
 
-      ## ----------------------------------------------------------------------
-
-      OUTPUT_A_GTR.$LF => sub {                   # Gather array/ref
-         my $_buf;
-
+      OUTPUT_A_GTR.$LF => sub {                   # Gather data
          chomp($_task_id = <$_DAU_R_SOCK>),
          chomp($_len     = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if ($_is_c_ref[$_task_id]) {
             local $_ = $self->{thaw}($_buf);
@@ -495,41 +506,19 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_S_GTR.$LF => sub {                   # Gather scalar
-         local $_;
-
-         chomp($_task_id = <$_DAU_R_SOCK>),
-         chomp($_len     = <$_DAU_R_SOCK>);
-
-         read $_DAU_R_SOCK, $_, $_len if ($_len >= 0);
-
-         if ($_is_c_ref[$_task_id]) {
-            $_gather[$_task_id]->($_);
-         }
-         elsif ($_is_h_ref[$_task_id]) {
-            $_gather[$_task_id]->{$_} = undef;
-         }
-         elsif ($_is_q_ref[$_task_id]) {
-            $_gather[$_task_id]->enqueue($_);
-         }
-         else {
-            push @{ $_gather[$_task_id] }, $_;
-         }
-
-         return;
-      },
-
       ## ----------------------------------------------------------------------
 
       OUTPUT_O_SND.$LF => sub {                   # Send >> STDOUT
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+
+         read $_DAU_R_SOCK, my($_buf), $_len;
+         $_buf = ${ $self->{thaw}($_buf) };
 
          if (defined $_user_output) {
             $_user_output->($_buf);
-         } else {
+         }
+         else {
+            use bytes;
             print {$_MCE_STDOUT} $_buf;
          }
 
@@ -537,14 +526,16 @@ sub _output_loop {
       },
 
       OUTPUT_E_SND.$LF => sub {                   # Send >> STDERR
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+
+         read $_DAU_R_SOCK, my($_buf), $_len;
+         $_buf = ${ $self->{thaw}($_buf) };
 
          if (defined $_user_error) {
             $_user_error->($_buf);
-         } else {
+         }
+         else {
+            use bytes;
             print {$_MCE_STDERR} $_buf;
          }
 
@@ -554,25 +545,29 @@ sub _output_loop {
       OUTPUT_F_SND.$LF => sub {                   # Send >> File
          my ($_buf, $_OUT_FILE);
 
-         chomp($_file = <$_DAU_R_SOCK>),
-         chomp($_len  = <$_DAU_R_SOCK>);
-
+         chomp($_len = <$_DAU_R_SOCK>);
          read $_DAU_R_SOCK, $_buf, $_len;
 
+         $_buf  = $self->{thaw}($_buf);
+         $_file = $_buf->[0];
+
          unless (exists $_sendto_fhs{$_file}) {
-            open $_sendto_fhs{$_file}, '>>', $_file
+            open $_sendto_fhs{$_file}, ">>", "$_file"
                or _croak "Cannot open file for writing ($_file): $!";
 
             binmode $_sendto_fhs{$_file};
 
-            ## Select new FH, turn on autoflush, restore the old FH.
-            if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_file}), $| = 1)[0]);
+            if (!exists $self->{flush_file} || $self->{flush_file}) {
+               local $!;
+               $_sendto_fhs{$_file}->autoflush(1);
             }
          }
 
-         $_OUT_FILE = $_sendto_fhs{$_file};
-         print {$_OUT_FILE} $_buf;
+         {
+            use bytes;
+            $_OUT_FILE = $_sendto_fhs{$_file};
+            print {$_OUT_FILE} $_buf->[1];
+         }
 
          return;
       },
@@ -580,39 +575,25 @@ sub _output_loop {
       OUTPUT_D_SND.$LF => sub {                   # Send >> File descriptor
          my ($_buf, $_OUT_FILE);
 
-         chomp($_fd  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
-
          read $_DAU_R_SOCK, $_buf, $_len;
 
-         unless (exists $_sendto_fhs{$_fd}) {
-            require IO::Handle unless (defined $IO::Handle::VERSION);
+         $_buf = $self->{thaw}($_buf);
 
-            $_sendto_fhs{$_fd} = IO::Handle->new();
-            $_sendto_fhs{$_fd}->fdopen($_fd, 'w')
-               or _croak "Cannot open file descriptor ($_fd): $!";
-
-            binmode $_sendto_fhs{$_fd};
-
-            ## Select new FH, turn on autoflush, restore the old FH.
-            if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_fd}), $| = 1)[0]);
-            }
+         {
+            use bytes;
+            $_OUT_FILE = _sendto_fhs_get($self, $_buf->[0]);
+            print {$_OUT_FILE} $_buf->[1];
          }
 
-         $_OUT_FILE = $_sendto_fhs{$_fd};
-         print {$_OUT_FILE} $_buf;
-
          return;
       },
 
       ## ----------------------------------------------------------------------
 
       OUTPUT_B_SYN.$LF => sub {                   # Barrier sync - begin
-
          if (!defined $_sync_cnt || $_sync_cnt == 0) {
-            $_syn_flag = 1;
-            $_sync_cnt = 0;
+            $_syn_flag = 1, $_sync_cnt = 0;
          }
 
          my $_total_running = ($_has_user_tasks)
@@ -620,7 +601,9 @@ sub _output_loop {
             : $self->{_total_running};
 
          if (++$_sync_cnt == $_total_running) {
-            for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF; }
+            for my $_i (1 .. $_total_running) {
+               syswrite($_BSB_W_SOCK, $LF);
+            }
             undef $_syn_flag;
          }
 
@@ -628,13 +611,95 @@ sub _output_loop {
       },
 
       OUTPUT_E_SYN.$LF => sub {                   # Barrier sync - end
-
          if (--$_sync_cnt == 0) {
             my $_total_running = ($_has_user_tasks)
                ? $self->{_task}->[0]->{_total_running}
                : $self->{_total_running};
 
-            for (1 .. $_total_running) { 1 until syswrite $_BSE_W_SOCK, $LF; }
+            for my $_i (1 .. $_total_running) {
+               syswrite($_BSB_R_SOCK, $LF);
+            }
+         }
+
+         return;
+      },
+
+      OUTPUT_S_IPC.$LF => sub {                   # Change to win32 IPC
+         syswrite($_DAT_R_SOCK, $LF);
+
+         $_win32_ipc = 1, goto _LOOP unless $_win32_ipc;
+
+         return;
+      },
+
+      OUTPUT_C_NFY.$LF => sub {                   # Chunk ID notification
+         chomp($_len = <$_DAU_R_SOCK>);
+
+         my ($_pid, $_chunk_id) = split /:/, $_len;
+         $self->{_pids_c}{$_pid} = $_chunk_id;
+
+         return;
+      },
+
+      OUTPUT_P_NFY.$LF => sub {                   # Progress notification
+         chomp($_len = <$_DAU_R_SOCK>);
+
+         $self->{progress}->( $_size_completed += $_len );
+
+         return;
+      },
+
+      OUTPUT_S_DIR.$LF => sub {                   # Make/get sess_dir
+         print {$_DAU_R_SOCK} $self->sess_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_T_DIR.$LF => sub {                   # Make/get tmp_dir
+         print {$_DAU_R_SOCK} $self->tmp_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_I_DLY.$LF => sub {                   # Interval delay
+         my $_tasks = $_has_user_tasks ? $self->{user_tasks} : undef;
+
+         chomp($_task_id = <$_DAU_R_SOCK>);
+
+         my $_interval = ($_tasks && exists $_tasks->[$_task_id]{interval})
+            ? $_tasks->[$_task_id]{interval}
+            : $self->{interval};
+
+         if ( $_interval ) {
+            my $_max_workers = ($_tasks)
+               ? $_tasks->[$_task_id]{max_workers}
+               : $self->{max_workers};
+
+            $_delay_wid[$_task_id] = 1
+               if (++$_delay_wid[$_task_id] > $_max_workers);
+
+            my $_nodes  = $_interval->{max_nodes};
+            my $_id     = $_interval->{node_id};
+            my $_delay  = $_interval->{delay} * $_nodes;
+
+            my $_app_tb = $_delay * $_max_workers;
+            my $_app_st = $_interval->{_time} + ($_delay / $_nodes * $_id);
+            my $_wrk_st = ($_delay_wid[$_task_id] - 1) * $_delay + $_app_st;
+
+            $_delay = $_wrk_st - MCE::Util::_time();
+
+            if ($_delay < 0.0 && $_app_tb) {
+               my $_count = int($_delay * -1 / $_app_tb + 0.5) + 1;
+               $_delay += $_app_tb * $_count;
+               $_interval->{_time} = MCE::Util::_time() if ($_count > 2e9);
+            }
+
+            ($_delay > 0.0)
+               ? print {$_DAU_R_SOCK} $_delay.$LF
+               : print {$_DAU_R_SOCK} '0'.$LF;
+         }
+         else {
+            print {$_DAU_R_SOCK} '0'.$LF;
          }
 
          return;
@@ -644,32 +709,41 @@ sub _output_loop {
 
    ## -------------------------------------------------------------------------
 
-   local ($!, $_);
+   local ($!, $?, $_);
 
+   $_aborted = $_chunk_id = $_eof_flag = $_size_completed = 0;
    $_has_user_tasks = (defined $self->{user_tasks}) ? 1 : 0;
    $_cs_one_flag = ($self->{chunk_size} == 1) ? 1 : 0;
-   $_aborted = $_chunk_id = $_eof_flag = 0;
 
    $_max_retries  = $self->{max_retries};
    $_on_post_exit = $self->{on_post_exit};
    $_on_post_run  = $self->{on_post_run};
    $_chunk_size   = $self->{chunk_size};
-   $_flush_file   = $self->{flush_file};
    $_user_output  = $self->{user_output};
    $_user_error   = $self->{user_error};
    $_single_dim   = $self->{_single_dim};
    $_sess_dir     = $self->{_sess_dir};
 
-   if ($_max_retries && !$_on_post_exit) {
+   if (defined $_max_retries && !$_on_post_exit) {
       $_on_post_exit = sub {
          my ($self, $_e, $_retry_cnt) = @_;
-         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: Chunk $_e->{id} failed");
 
-         ($_retry_cnt < $_max_retries)
-            ? print {*STDERR} "$_msg, retrying chunk attempt #${_cnt}\n"
-            : print {*STDERR} "$_msg\n";
+         if ($_e->{id}) {
+            my $_cnt = $_retry_cnt + 1;
+            my $_msg = "Error: chunk $_e->{id} failed";
 
-         $self->restart_worker;
+            if (defined $self->{init_relay}) {
+               print {*STDERR} "$_msg, retrying chunk attempt # $_cnt\n"
+                  if ($_retry_cnt < $_max_retries);
+            }
+            else {
+               ($_retry_cnt < $_max_retries)
+                  ? print {*STDERR} "$_msg, retrying chunk attempt # $_cnt\n"
+                  : print {*STDERR} "$_msg\n";
+            }
+
+            $self->restart_worker;
+         }
       };
    }
 
@@ -705,7 +779,12 @@ sub _output_loop {
    if (defined $_input_data && ref $_input_data eq 'ARRAY') {
       $_input_size = @{ $_input_data };
       $_offset_pos = 0;
-   } else {
+   }
+   elsif (defined $_input_data && ref $_input_data eq 'HASH') {
+      $_input_size = scalar( keys %{ $_input_data } );
+      $_offset_pos = 0;
+   }
+   else {
       $_input_size = $_offset_pos = 0;
    }
 
@@ -718,7 +797,6 @@ sub _output_loop {
    }
    else {
       $_MCE_STDOUT = \*STDOUT;
-      binmode $_MCE_STDOUT;
    }
 
    if (defined $self->{stderr_file}) {
@@ -728,29 +806,36 @@ sub _output_loop {
    }
    else {
       $_MCE_STDERR = \*STDERR;
-      binmode $_MCE_STDERR;
    }
 
-   ## Make MCE_STDOUT the default handle.
-   ## Flush STDERR/STDOUT handles if requested.
+   ## Autoflush STDERR-STDOUT handles if not specified or requested.
 
-   my $_old_hndl = select $_MCE_STDOUT;
+   {
+      local $!;
 
-   if ($self->{flush_stdout}) {
-      local $|; select((select($_MCE_STDOUT), $| = 1)[0]);
-   }
-   if ($self->{flush_stderr}) {
-      local $|; select((select($_MCE_STDERR), $| = 1)[0]);
+      $_MCE_STDERR->autoflush(1)
+         if ( !exists $self->{flush_stderr} || $self->{flush_stderr} );
+
+      $_MCE_STDOUT->autoflush(1)
+         if ( !exists $self->{flush_stdout} || $self->{flush_stdout} );
    }
 
    ## -------------------------------------------------------------------------
 
    ## Output event loop.
 
-   my $_func; my $_channels = $self->{_dat_r_sock};
+   my $_channels = $self->{_dat_r_sock};
+   my $_func;
+
+   $_win32_ipc = (
+      $ENV{'PERL_MCE_IPC'} eq 'win32' ||
+      defined($self->{max_retries}) ||
+      $INC{'MCE/Child.pm'} ||
+      $INC{'MCE/Hobo.pm'}
+   );
 
    $_BSB_W_SOCK = $self->{_bsb_w_sock};
-   $_BSE_W_SOCK = $self->{_bse_w_sock};
+   $_BSB_R_SOCK = $self->{_bsb_r_sock};
    $_DAT_R_SOCK = $self->{_dat_r_sock}->[0];
 
    $_RS     = $self->{RS} || $/;
@@ -770,86 +855,126 @@ sub _output_loop {
    ## Wait on requests *with* timeout capability. Exit loop when all workers
    ## have completed processing or exited prematurely.
 
-   if ($self->{loop_timeout} && $self->{_pids} && $^O ne 'MSWin32') {
+   _LOOP:
+
+   if ($self->{loop_timeout} && @{ $self->{_tids} } == 0 && $^O ne 'MSWin32') {
       my ($_list, $_timeout) = ($self->{_pids}, $self->{loop_timeout});
       my ($_DAT_W_SOCK, $_pid) = ($self->{_dat_w_sock}->[0]);
 
+      $self->{_pids_c} = {};  # Chunk ID notification
+
       $_timeout = 5 if $_timeout < 5;
 
       local $SIG{ALRM} = sub {
-         alarm 0;
+         alarm 0; local ($!, $?);
+
          for my $i (0 .. @{ $_list }) {
             if ($_pid = $_list->[$i]) {
-               if (waitpid($_pid, 1)) {
+               if (waitpid($_pid, _WNOHANG)) {
+
+                  $_list->[$i] = undef;
+
+                  if ($? > abs($self->{_wrk_status})) {
+                     $self->{_wrk_status} = $?;
+                  }
+
+                  my $_task_id = $self->{_pids_t}{$_pid};
+                  my $_wid     = $self->{_pids_w}{$_pid};
+
                   $self->{_total_exited}  += 1;
                   $self->{_total_running} -= 1;
                   $self->{_total_workers} -= 1;
-                  $_list->[$i] = undef;
+
+                  if ($_has_user_tasks && $_task_id >= 0) {
+                     $self->{_task}->[$_task_id]->{_total_running} -= 1;
+                     $self->{_task}->[$_task_id]->{_total_workers} -= 1;
+                  }
+
+                  my $_total_running = ($_has_user_tasks)
+                     ? $self->{_task}->[$_task_id]->{_total_running}
+                     : $self->{_total_running};
+
+                  if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
+                     if ($_sync_cnt == $_total_running) {
+                        for my $_i (1 .. $_total_running) {
+                           syswrite($_BSB_W_SOCK, $LF);
+                        }
+                        undef $_syn_flag;
+                     }
+                  }
+
+                  _task_end($self, $_task_id) unless $_total_running;
+
+                  if (my $_cid = $self->{_pids_c}{$_pid}) {
+                     warn "Error: process $_pid has ended prematurely\n",
+                          "Error: chunk $_cid failed\n";
+
+                     if ($_cid > $self->{_relayed}) {
+                        local $SIG{CHLD} = 'IGNORE';
+                        my $_pid = fork;
+
+                        if (defined $_pid && $_pid == 0) {
+                           delete $self->{max_retries};
+
+                           $self->{_chunk_id} = $_cid;
+                           $self->{_task_id}  = $_task_id;
+                           $self->{_wid}      = $_wid;
+
+                           eval 'MCE::relay';
+
+                           CORE::kill('KILL', $$);
+                           CORE::exit(0);
+                        }
+                     }
+                  }
+
+                  delete $self->{_pids_c}{$_pid};
+                  delete $self->{_pids_t}{$_pid};
+                  delete $self->{_pids_w}{$_pid};
                }
             }
          }
+
          print {$_DAT_W_SOCK} 'NOOP'.$LF . '0'.$LF;
       };
 
       while ( $self->{_total_running} ) {
-         alarm $_timeout;
-         $_func = <$_DAT_R_SOCK>;
+         alarm $_timeout; $_func = <$_DAT_R_SOCK>; alarm 0;
          $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
 
-         alarm 0;
          if (exists $_core_output_function{$_func}) {
             $_core_output_function{$_func}();
          } elsif (exists $_plugin_function->{$_func}) {
             $_plugin_function->{$_func}();
          }
       }
+
+      delete $self->{_pids_c};
    }
 
    ## Wait on requests *without* timeout capability.
 
    elsif ($^O eq 'MSWin32') {
-      # The normal loop hangs on Windows when processes/threads start/exit.
-      # Using ioctl() properly, http://www.perlmonks.org/?node_id=780083
+      MCE::Util::_nonblocking($_DAT_R_SOCK, 1) if $_win32_ipc;
 
-      my ($_bytes, $_cnt, $_retries) = ("\x00\x00\x00\x00", 1, 0);
-      my ($_ptr_bytes, $_nbytes) = (unpack('I', pack('P', $_bytes)));
+      while ($self->{_total_running}) {
+         MCE::Util::_sysread2($_DAT_R_SOCK, $_func, 8);
+         last() unless length($_func) == 8;
+         $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
 
-      while (1) {
-         ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);  # MSWin32 FIONREAD
-
-         unless ($_nbytes = unpack('I', $_bytes)) {
-            # delay to not consume a CPU from non-blocking ioctl
-            if ($_cnt) {
-               if (++$_cnt > 900) {
-                  $_cnt = 1, sleep 0.015;
-                  $_cnt = 0 if ++$_retries == 2;
-               }
-               next;
-            }
-            sleep 0.030;
-            next;
+         if (exists $_core_output_function{$_func}) {
+            $_core_output_function{$_func}();
+         } elsif (exists $_plugin_function->{$_func}) {
+            $_plugin_function->{$_func}();
          }
-
-         $_cnt = 1, $_retries = 0;
-
-         do {
-            sysread($_DAT_R_SOCK, $_func, 8);
-            $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
-
-            if (exists $_core_output_function{$_func}) {
-               $_core_output_function{$_func}();
-            } elsif (exists $_plugin_function->{$_func}) {
-               $_plugin_function->{$_func}();
-            }
-
-         } while (($_nbytes -= 8) >= 8);
-
-         last unless $self->{_total_running};
       }
+
+      MCE::Util::_nonblocking($_DAT_R_SOCK, 0) if $_win32_ipc;
    }
    else {
       while ($self->{_total_running}) {
          $_func = <$_DAT_R_SOCK>;
+         last() unless length($_func) == 6;
          $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
 
          if (exists $_core_output_function{$_func}) {
@@ -872,15 +997,9 @@ sub _output_loop {
 
    ## Close opened sendto file handles.
 
-   for my $_p (keys %_sendto_fhs) {
-      close  $_sendto_fhs{$_p};
-      undef  $_sendto_fhs{$_p};
-      delete $_sendto_fhs{$_p};
-   }
+   _sendto_fhs_close();
 
-   ## Restore the default handle. Close MCE STDOUT/STDERR handles.
-
-   select $_old_hndl;
+   ## Close MCE STDOUT/STDERR handles.
 
    eval q{
       close $_MCE_STDOUT if (fileno $_MCE_STDOUT > 2);
@@ -892,3 +1011,32 @@ sub _output_loop {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Manager - Core methods for the manager process
+
+=head1 VERSION
+
+This document describes MCE::Core::Manager version 1.867_01
+
+=head1 DESCRIPTION
+
+This package provides the loop and relevant methods used internally by the
+manager process.
+
+There is no public API.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
