@@ -14,7 +14,7 @@ package MCE::Core::Validation;
 use strict;
 use warnings;
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## Items below are folded into MCE.
 
@@ -42,55 +42,20 @@ sub _validate_args {
          unless (-e $_s->{input_data});
    }
 
-   _croak("$_tag: (use_slurpio) is not 0 or 1")
-      if ($_s->{use_slurpio} && $_s->{use_slurpio} !~ /\A[01]\z/);
-   _croak("$_tag: (parallel_io) is not 0 or 1")
-      if ($_s->{parallel_io} && $_s->{parallel_io} !~ /\A[01]\z/);
-   _croak("$_tag: (posix_exit) is not 0 or 1")
-      if ($_s->{posix_exit} && $_s->{posix_exit} !~ /\A[01]\z/);
-
-   _croak("$_tag: (job_delay) is not valid")
-      if ($_s->{job_delay} && (!looks_like_number($_s->{job_delay}) ||
-         $_s->{job_delay} < 0));
-   _croak("$_tag: (spawn_delay) is not valid")
-      if ($_s->{spawn_delay} && (!looks_like_number($_s->{spawn_delay}) ||
-         $_s->{spawn_delay} < 0));
-   _croak("$_tag: (submit_delay) is not valid")
-      if ($_s->{submit_delay} && (!looks_like_number($_s->{submit_delay}) ||
-         $_s->{submit_delay} < 0));
-
-   _croak("$_tag: (freeze) is not a CODE reference")
-      if ($_s->{freeze} && ref $_s->{freeze} ne 'CODE');
-   _croak("$_tag: (thaw) is not a CODE reference")
-      if ($_s->{thaw} && ref $_s->{thaw} ne 'CODE');
-
-   _croak("$_tag: (on_post_exit) is not a CODE reference")
-      if ($_s->{on_post_exit} && ref $_s->{on_post_exit} ne 'CODE');
-   _croak("$_tag: (on_post_run) is not a CODE reference")
-      if ($_s->{on_post_run} && ref $_s->{on_post_run} ne 'CODE');
-   _croak("$_tag: (user_error) is not a CODE reference")
-      if ($_s->{user_error} && ref $_s->{user_error} ne 'CODE');
-   _croak("$_tag: (user_output) is not a CODE reference")
-      if ($_s->{user_output} && ref $_s->{user_output} ne 'CODE');
-
-   _croak("$_tag: (flush_file) is not 0 or 1")
-      if ($_s->{flush_file} && $_s->{flush_file} !~ /\A[01]\z/);
-   _croak("$_tag: (flush_stderr) is not 0 or 1")
-      if ($_s->{flush_stderr} && $_s->{flush_stderr} !~ /\A[01]\z/);
-   _croak("$_tag: (flush_stdout) is not 0 or 1")
-      if ($_s->{flush_stdout} && $_s->{flush_stdout} !~ /\A[01]\z/);
-
-   _croak("$_tag: (loop_timeout) is not valid")
-      if ($_s->{loop_timeout} && !looks_like_number($_s->{loop_timeout}));
+   for my $_k (qw(job_delay spawn_delay submit_delay loop_timeout)) {
+      _croak("$_tag: ($_k) is not valid")
+         if ($_s->{$_k} && (!looks_like_number($_s->{$_k}) || $_s->{$_k} < 0));
+   }
+   for my $_k (qw(freeze thaw on_post_exit on_post_run user_error user_output)) {
+      _croak("$_tag: ($_k) is not a CODE reference")
+         if ($_s->{$_k} && ref $_s->{$_k} ne 'CODE');
+   }
 
    _validate_args_s($_s);
 
    if (defined $_s->{user_tasks}) {
       for my $_t (@{ $_s->{user_tasks} }) {
          _validate_args_s($_s, $_t);
-
-         _croak("$_tag: (task_end) is not a CODE reference")
-            if ($_t->{task_end} && ref $_t->{task_end} ne 'CODE');
       }
    }
 
@@ -113,6 +78,7 @@ sub _validate_args_s {
 
    if (defined $_s->{max_workers}) {
       $_s->{max_workers} = _parse_max_workers($_s->{max_workers});
+
       _croak("$_tag: (max_workers) is not valid")
          if ($_s->{max_workers} !~ /\A\d+\z/);
    }
@@ -126,28 +92,24 @@ sub _validate_args_s {
       }
 
       _croak("$_tag: (chunk_size) is not valid")
-         if ($_s->{chunk_size} !~ /\A\d+\z/ or $_s->{chunk_size} == 0);
+         if ($_s->{chunk_size} !~ /\A[0-9e\+]+\z/ or $_s->{chunk_size} == 0);
 
-      $_s->{chunk_size} = MAX_CHUNK_SIZE
-         if ($_s->{chunk_size} > MAX_CHUNK_SIZE);
+      $_s->{chunk_size} = int($_s->{chunk_size});
    }
 
    _croak("$_tag: (RS) is not valid")
       if ($_s->{RS} && ref $_s->{RS} ne '');
    _croak("$_tag: (max_retries) is not valid")
       if ($_s->{max_retries} && $_s->{max_retries} !~ /\A\d+\z/);
-   _croak("$_tag: (use_threads) is not 0 or 1")
-      if ($_s->{use_threads} && $_s->{use_threads} !~ /\A[01]\z/);
 
-   _croak("$_tag: (user_begin) is not a CODE reference")
-      if ($_s->{user_begin} && ref $_s->{user_begin} ne 'CODE');
-   _croak("$_tag: (user_func) is not a CODE reference")
-      if ($_s->{user_func} && ref $_s->{user_func} ne 'CODE');
-   _croak("$_tag: (user_end) is not a CODE reference")
-      if ($_s->{user_end} && ref $_s->{user_end} ne 'CODE');
+   for my $_k (qw(progress user_begin user_end user_func task_end)) {
+      _croak("$_tag: ($_k) is not a CODE reference")
+         if ($_s->{$_k} && ref $_s->{$_k} ne 'CODE');
+   }
 
    if (defined $_s->{gather}) {
       my $_ref = ref $_s->{gather};
+
       _croak("$_tag: (gather) is not a valid reference")
          if ( $_ref ne 'MCE::Queue' && $_ref ne 'Thread::Queue' &&
               $_ref ne 'ARRAY' && $_ref ne 'HASH' && $_ref ne 'CODE' );
@@ -167,10 +129,10 @@ sub _validate_args_s {
             if (ref $_seq ne 'HASH');
       }
 
-      _croak("$_tag: (begin) is not defined for sequence")
-         unless (defined $_seq->{begin});
-      _croak("$_tag: (end) is not defined for sequence")
-         unless (defined $_seq->{end});
+      for my $_k (qw(begin end)) {
+         _croak("$_tag: ($_k) is not defined for sequence")
+            unless (defined $_seq->{$_k});
+      }
 
       for my $_p (qw(begin end step)) {
          _croak("$_tag: ($_p) is not valid for sequence")
@@ -178,12 +140,25 @@ sub _validate_args_s {
       }
 
       unless (defined $_seq->{step}) {
-         $_seq->{step} = ($_seq->{begin} < $_seq->{end}) ? 1 : -1;
+         $_seq->{step} = ($_seq->{begin} <= $_seq->{end}) ? 1 : -1;
          if (ref $_s->{sequence} eq 'ARRAY') {
             $_s->{sequence}->[2] = $_seq->{step};
          }
       }
 
+      if (ref $_s->{sequence} eq 'HASH') {
+         for my $_k ('begin', 'end', 'step') {
+            $_s->{sequence}{$_k} = int($_s->{sequence}{$_k})
+               unless ($_s->{sequence}{$_k} =~ /\./);
+         }
+      }
+      else {
+         for my $_i (0, 1, 2) {
+            $_s->{sequence}[$_i] = int($_s->{sequence}[$_i])
+               unless ($_s->{sequence}[$_i] =~ /\./);
+         }
+      }
+
       if ( ($_seq->{step} < 0 && $_seq->{begin} < $_seq->{end}) ||
            ($_seq->{step} > 0 && $_seq->{begin} > $_seq->{end}) ||
            ($_seq->{step} == 0)
@@ -217,7 +192,7 @@ sub _validate_args_s {
 
       $_i->{max_nodes} = 1 unless (exists $_i->{max_nodes});
       $_i->{node_id}   = 1 unless (exists $_i->{node_id});
-      $_i->{_time}     = time;
+      $_i->{_time}     = MCE::Util::_time();
    }
 
    return;
@@ -245,5 +220,189 @@ sub _validate_runstate {
    return;
 }
 
+###############################################################################
+## ----------------------------------------------------------------------------
+## Private functions for MCE Models { Flow, Grep, Loop, Map, Step, Stream }.
+##
+###############################################################################
+
+sub _parse_chunk_size {
+
+   my ($_chunk_size, $_max_workers, $_params, $_input_data, $_array_size) = @_;
+
+   @_ = ();
+
+   return $_chunk_size if (!defined $_chunk_size || !defined $_max_workers);
+
+   if (defined $_params && exists $_params->{chunk_size}) {
+      $_chunk_size = $_params->{chunk_size};
+   }
+
+   if ($_chunk_size =~ /([0-9\.]+)K\z/i) {
+      $_chunk_size = int($1 * 1024 + 0.5);
+   }
+   elsif ($_chunk_size =~ /([0-9\.]+)M\z/i) {
+      $_chunk_size = int($1 * 1024 * 1024 + 0.5);
+   }
+
+   if ($_chunk_size eq 'auto') {
+
+      if ( (defined $_params && ref $_params->{input_data} eq 'CODE') ||
+           (defined $_input_data && ref $_input_data eq 'CODE')
+      ) {
+         # Iterators may optionally use chunk_size to determine how much
+         # to return per iteration. The default is 1 for MCE Models, same
+         # as for the Core API. The user_func receives an array_ref
+         # regardless if 1 or greater.
+         #
+         # sub make_iter {
+         #    ...
+         #    return sub {
+         #       my ($chunk_size) = @_;
+         #       ...
+         #    };
+         # }
+         return 1;
+      }
+
+      my $_is_file;
+      my $_size = $_array_size;
+
+      if (defined $_input_data) {
+         if (ref $_input_data eq 'ARRAY') {
+            $_size = scalar @{ $_input_data };
+         } elsif (ref $_input_data eq 'HASH') {
+            $_size = scalar keys %{ $_input_data };
+         }
+      }
+
+      if (defined $_params && exists $_params->{sequence}) {
+         my ($_begin, $_end, $_step);
+
+         if (ref $_params->{sequence} eq 'HASH') {
+            $_begin = $_params->{sequence}->{begin};
+            $_end   = $_params->{sequence}->{end};
+            $_step  = $_params->{sequence}->{step} || 1;
+         }
+         else {
+            $_begin = $_params->{sequence}[0];
+            $_end   = $_params->{sequence}[1];
+            $_step  = $_params->{sequence}[2] || 1;
+         }
+
+         if (!defined $_input_data && !$_array_size) {
+            $_size = abs($_end - $_begin) / $_step + 1;
+         }
+      }
+      elsif (defined $_params && exists $_params->{_file}) {
+         my $_ref = ref $_params->{_file};
+
+         if ($_ref eq 'SCALAR') {
+            $_size = length ${ $_params->{_file} };
+         } elsif ($_ref eq '') {
+            $_size = -s $_params->{_file};
+         } else {
+            $_size = 0; $_chunk_size = 393_216;  # 384K
+         }
+
+         $_is_file = 1;
+      }
+      elsif (defined $_input_data) {
+         if (ref($_input_data) =~ /^(?:GLOB|FileHandle|IO::)/) {
+            $_is_file = 1; $_size = 0; $_chunk_size = 393_216;  # 384K
+         }
+         elsif (ref $_input_data eq 'SCALAR') {
+            $_is_file = 1; $_size = length ${ $_input_data };
+         }
+      }
+
+      if (defined $_is_file) {
+         if ($_size) {
+            $_chunk_size = int($_size / $_max_workers / 24 + 0.5);
+            $_chunk_size = 5_242_880 if $_chunk_size > 5_242_880;  # 5M
+            $_chunk_size = 2 if $_chunk_size <= 8192;
+         }
+      }
+      else {
+         $_chunk_size = int($_size / $_max_workers / 24 + 0.5);
+         $_chunk_size = 8000 if $_chunk_size > 8000;
+         $_chunk_size = 2 if $_chunk_size < 2;
+      }
+   }
+
+   return $_chunk_size;
+}
+
+sub _parse_max_workers {
+
+   my ($_max_workers) = @_;
+
+   @_ = ();
+
+   return $_max_workers unless (defined $_max_workers);
+
+   if ($_max_workers =~ /^auto(?:$|\s*([\-\+\/\*])\s*(.+)$)/i) {
+      my ($_ncpu_ul, $_ncpu);
+
+      $_ncpu_ul = $_ncpu = MCE::Util::get_ncpu();
+      $_ncpu_ul = 8 if ($_ncpu_ul > 8);
+
+      if ($1 && $2) {
+         local $@; $_max_workers = eval "int($_ncpu_ul $1 $2 + 0.5)"; ## no critic
+         $_max_workers = 1 if (!$_max_workers || $_max_workers < 1);
+         $_max_workers = $_ncpu if ($_max_workers > $_ncpu);
+      }
+      else {
+         $_max_workers = $_ncpu_ul;
+      }
+   }
+
+   return $_max_workers;
+}
+
+sub _validate_number {
+
+   my ($_n, $_key, $_tag) = @_;
+
+   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
+
+   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
+
+   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
+      _croak("$_tag: ($_key) is not valid");
+   }
+
+   return;
+}
+
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Validation - Core validation methods for Many-Core Engine
+
+=head1 VERSION
+
+This document describes MCE::Core::Validation version 1.867_01
+
+=head1 DESCRIPTION
+
+This package provides validation methods used internally by the manager
+process.
+
+There is no public API.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
