@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -22,12 +22,10 @@ use MCE;
 
 our @CARP_NOT = qw( MCE );
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
-my $_caller;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ###############################################################################
@@ -63,7 +61,7 @@ sub import {
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
 
-      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
          if ( shift eq '0' ) {
             require Storable;
@@ -76,19 +74,15 @@ sub import {
       _croak("Error: ($_argument) invalid module option");
    }
 
-   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+   $_p->{MAX_WORKERS} = MCE::_parse_max_workers($_p->{MAX_WORKERS});
 
-   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
-   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+   MCE::_validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS', $_tag);
+   MCE::_validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE', $_tag)
       unless ($_p->{CHUNK_SIZE} eq 'auto');
 
    return;
 }
 
-END {
-   %{ $_MCE } = ();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Gather callback for storing by chunk_id => chunk_ref into a hash.
@@ -116,10 +110,13 @@ sub _gather {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
-   MCE::Grep->finish( my $_pkg = "$$.$_tid.".caller() );
+   my $_pkg = "$$.$_tid.".caller();
 
    $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
+   _croak("$_tag: (HASH) not allowed as input by this MCE model")
+      if ( ref $_params->{$_pkg}{input_data} eq 'HASH' );
+
    @_ = ();
 
    return;
@@ -130,19 +127,19 @@ sub finish (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
    my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
 
-   if ( $_pkg eq 'MCE::Shared::Server' ) {
-      MCE::Grep->finish($_) for ( keys %{ $_MCE } );
-      %{ $_MCE } = ();
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Grep->finish($_k, 1); }
    }
-   elsif ( exists $_MCE->{$_pkg} ) {
-      MCE::_save_state(), $_MCE->{$_pkg}->shutdown(@_), MCE::_restore_state()
-         if $_MCE->{$_pkg}{_spawned};
-
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
       $_total_chunks = undef, undef %_tmp;
 
       delete $_prev_c->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
+   @_ = ();
+
    return;
 }
 
@@ -157,7 +154,7 @@ sub run_file (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift; my $_file = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -173,7 +170,7 @@ sub run_file (&@) {
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
       $_params->{$_pid}{_file} = $_file;
    }
    else {
@@ -196,7 +193,7 @@ sub run_seq (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -216,7 +213,7 @@ sub run_seq (&@) {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
       $_params->{$_pid}{sequence} = $_[0];
    }
-   elsif (ref $_[0] eq '') {
+   elsif (ref $_[0] eq '' || ref($_[0]) =~ /^Math::/) {
       $_begin = $_[0]; $_end = $_[1];
       $_params->{$_pid}{sequence} = [ @_ ];
    }
@@ -229,7 +226,7 @@ sub run_seq (&@) {
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{$_pid}{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = undef;
 
    @_ = ();
 
@@ -247,18 +244,20 @@ sub run (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift;  $_total_chunks = 0; undef %_tmp;
-   my $_pkg  = defined $_caller ? $_caller : caller();  $_caller = undef;
+   my $_pkg  = caller() eq 'MCE::Grep' ? caller(1) : caller();
    my $_pid  = "$$.$_tid.$_pkg";
 
    my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
    my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+   if (@_ == 1 && $_r =~ /^(?:ARRAY|HASH|SCALAR|CODE|GLOB|FileHandle|IO::)/) {
+      _croak("$_tag: (HASH) not allowed as input by this MCE model")
+         if $_r eq 'HASH';
       $_input_data = shift;
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
-      $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
+      $_max_workers = MCE::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
       delete $_p->{sequence}    if (defined $_input_data || scalar @_);
@@ -269,7 +268,7 @@ sub run (&@) {
       delete $_p->{gather}      if (exists $_p->{gather});
    }
 
-   my $_chunk_size = MCE::Util::_parse_chunk_size(
+   my $_chunk_size = MCE::_parse_chunk_size(
       $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
       $_input_data, scalar @_
    );
@@ -284,7 +283,7 @@ sub run (&@) {
 
    ## -------------------------------------------------------------------------
 
-   MCE::_save_state();
+   MCE::_save_state($_MCE->{$_pid});
 
    if (!defined $_prev_c->{$_pid} || $_prev_c->{$_pid} != $_code) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
@@ -348,7 +347,7 @@ sub run (&@) {
             next if ($_k eq 'input_data');
             next if ($_k eq 'chunk_size');
 
-            _croak("MCE::Grep: ($_k) is not a valid constructor argument")
+            _croak("$_tag: ($_k) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_k});
 
             $_opts{$_k} = $_p->{$_k};
@@ -396,10 +395,6 @@ sub run (&@) {
       }
    }
 
-   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
-      $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
-   }
-
    MCE::_restore_state();
 
    if ($_wantarray) {
@@ -423,21 +418,6 @@ sub _croak {
    goto &MCE::_croak;
 }
 
-sub _validate_number {
-
-   my ($_n, $_key) = @_;
-
-   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
-
-   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
-
-   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
-      _croak("$_tag: ($_key) is not valid");
-   }
-
-   return;
-}
-
 1;
 
 __END__
@@ -454,29 +434,37 @@ MCE::Grep - Parallel grep model similar to the native grep function
 
 =head1 VERSION
 
-This document describes MCE::Grep version 1.799_03
+This document describes MCE::Grep version 1.867_01
 
 =head1 SYNOPSIS
 
-   ## Exports mce_grep, mce_grep_f, and mce_grep_s
-   use MCE::Grep;
+ ## Exports mce_grep, mce_grep_f, and mce_grep_s
+ use MCE::Grep;
 
-   ## Array or array_ref
-   my @a = mce_grep { $_ % 5 == 0 } 1..10000;
-   my @b = mce_grep { $_ % 5 == 0 } [ 1..10000 ];
+ ## Array or array_ref
+ my @a = mce_grep { $_ % 5 == 0 } 1..10000;
+ my @b = mce_grep { $_ % 5 == 0 } \@list;
 
-   ## File_path, glob_ref, or scalar_ref
-   my @c = mce_grep_f { /pattern/ } "/path/to/file";
-   my @d = mce_grep_f { /pattern/ } $file_handle;
-   my @e = mce_grep_f { /pattern/ } \$scalar;
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_grep { $_->[1] % 2 == 0 } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_grep { $_->[1] % 2 == 0 } \@deeply_list;
 
-   ## Sequence of numbers (begin, end [, step, format])
-   my @f = mce_grep_s { %_ * 3 == 0 } 1, 10000, 5;
-   my @g = mce_grep_s { %_ * 3 == 0 } [ 1, 10000, 5 ];
+ ## File path, glob ref, IO::All::{ File, Pipe, STDIO } obj, or scalar ref
+ ## Workers read directly and not involve the manager process
+ my @e = mce_grep_f { /pattern/ } "/path/to/file"; # efficient
 
-   my @h = mce_grep_s { %_ * 3 == 0 } {
-      begin => 1, end => 10000, step => 5, format => undef
-   };
+ ## Involves the manager process, therefore slower
+ my @f = mce_grep_f { /pattern/ } $file_handle;
+ my @g = mce_grep_f { /pattern/ } $io;
+ my @h = mce_grep_f { /pattern/ } \$scalar;
+
+ ## Sequence of numbers (begin, end [, step, format])
+ my @i = mce_grep_s { %_ * 3 == 0 } 1, 10000, 5;
+ my @j = mce_grep_s { %_ * 3 == 0 } [ 1, 10000, 5 ];
+
+ my @k = mce_grep_s { %_ * 3 == 0 } {
+    begin => 1, end => 10000, step => 5, format => undef
+ };
 
 =head1 DESCRIPTION
 
@@ -485,16 +473,16 @@ MCE incurs a small overhead due to passing of data. A fast code block will
 run faster natively. However, the overhead will likely diminish as the
 complexity increases for the code.
 
-   my @m1 =     grep { $_ % 5 == 0 } 1..1000000;          ## 0.065 secs
-   my @m2 = mce_grep { $_ % 5 == 0 } 1..1000000;          ## 0.194 secs
+ my @m1 =     grep { $_ % 5 == 0 } 1..1000000;          ## 0.065 secs
+ my @m2 = mce_grep { $_ % 5 == 0 } 1..1000000;          ## 0.194 secs
 
 Chunking, enabled by default, greatly reduces the overhead behind the scene.
 The time for mce_grep below also includes the time for data exchanges between
 the manager and worker processes. More parallelization will be seen when the
 code incurs additional CPU time.
 
-   my @m1 =     grep { /[2357][1468][9]/ } 1..1000000;    ## 0.353 secs
-   my @m2 = mce_grep { /[2357][1468][9]/ } 1..1000000;    ## 0.218 secs
+ my @m1 =     grep { /[2357][1468][9]/ } 1..1000000;    ## 0.353 secs
+ my @m2 = mce_grep { /[2357][1468][9]/ } 1..1000000;    ## 0.218 secs
 
 Even faster is mce_grep_s; useful when input data is a range of numbers.
 Workers generate sequences mathematically among themselves without any
@@ -502,48 +490,48 @@ interaction from the manager process. Two arguments are required for
 mce_grep_s (begin, end). Step defaults to 1 if begin is smaller than end,
 otherwise -1.
 
-   my @m3 = mce_grep_s { /[2357][1468][9]/ } 1, 1000000;  ## 0.165 secs
+ my @m3 = mce_grep_s { /[2357][1468][9]/ } 1, 1000000;  ## 0.165 secs
 
 Although this document is about MCE::Grep, the L<MCE::Stream> module can write
 results immediately without waiting for all chunks to complete. This is made
 possible by passing the reference to an array (in this case @m4 and @m5).
 
-   use MCE::Stream default_mode => 'grep';
+ use MCE::Stream default_mode => 'grep';
 
-   my @m4; mce_stream \@m4, sub { /[2357][1468][9]/ }, 1..1000000;
+ my @m4; mce_stream \@m4, sub { /[2357][1468][9]/ }, 1..1000000;
 
-      ## Completed in 0.203 secs. This is amazing considering the
-      ## overhead for passing data between the manager and workers.
+    ## Completed in 0.203 secs. This is amazing considering the
+    ## overhead for passing data between the manager and workers.
 
-   my @m5; mce_stream_s \@m5, sub { /[2357][1468][9]/ }, 1, 1000000;
+ my @m5; mce_stream_s \@m5, sub { /[2357][1468][9]/ }, 1, 1000000;
 
-      ## Completed in 0.120 secs. Like with mce_grep_s, specifying a
-      ## sequence specification turns out to be faster due to lesser
-      ## overhead for the manager process.
+    ## Completed in 0.120 secs. Like with mce_grep_s, specifying a
+    ## sequence specification turns out to be faster due to lesser
+    ## overhead for the manager process.
 
 A common scenario is grepping for pattern(s) inside a massive log file.
 Notice how parallelism increases as complexity increases for the pattern.
 Testing was done against a 300 MB file containing 250k lines.
 
-   use MCE::Grep;
+ use MCE::Grep;
 
-   my @m; open my $LOG, "<", "/path/to/log/file" or die "$!\n";
+ my @m; open my $LOG, "<", "/path/to/log/file" or die "$!\n";
 
-   @m = grep { /pattern/ } <$LOG>;                      ##  0.756 secs
-   @m = grep { /foobar|[2357][1468][9]/ } <$LOG>;       ## 24.681 secs
+ @m = grep { /pattern/ } <$LOG>;                      ##  0.756 secs
+ @m = grep { /foobar|[2357][1468][9]/ } <$LOG>;       ## 24.681 secs
 
-   ## Parallelism with mce_grep. This involves the manager process
-   ## due to processing a file handle.
+ ## Parallelism with mce_grep. This involves the manager process
+ ## due to processing a file handle.
 
-   @m = mce_grep { /pattern/ } <$LOG>;                  ##  0.997 secs
-   @m = mce_grep { /foobar|[2357][1468][9]/ } <$LOG>;   ##  7.439 secs
+ @m = mce_grep { /pattern/ } <$LOG>;                  ##  0.997 secs
+ @m = mce_grep { /foobar|[2357][1468][9]/ } <$LOG>;   ##  7.439 secs
 
-   ## Even faster with mce_grep_f. Workers access the file directly
-   ## with zero interaction from the manager process.
+ ## Even faster with mce_grep_f. Workers access the file directly
+ ## with zero interaction from the manager process.
 
-   my $LOG = "/path/to/file";
-   @m = mce_grep_f { /pattern/ } $LOG;                  ##  0.112 secs
-   @m = mce_grep_f { /foobar|[2357][1468][9]/ } $LOG;   ##  6.840 secs
+ my $LOG = "/path/to/file";
+ @m = mce_grep_f { /pattern/ } $LOG;                  ##  0.112 secs
+ @m = mce_grep_f { /foobar|[2357][1468][9]/ } $LOG;   ##  6.840 secs
 
 =head1 PARSING HUGE FILES
 
@@ -552,71 +540,68 @@ is found from not knowing the pattern inside the code block. Use the following
 snippet as a template to achieve better performance. Also, take a look at
 examples/egrep.pl, included with the distribution.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      max_workers => 8, use_slurpio => 1
-   };
+ MCE::Loop::init {
+    max_workers => 8, use_slurpio => 1
+ };
 
-   my $pattern  = 'karl';
-   my $hugefile = 'very_huge.file';
+ my $pattern  = 'karl';
+ my $hugefile = 'very_huge.file';
 
-   my @result = mce_loop_f {
-      my ($mce, $slurp_ref, $chunk_id) = @_;
+ my @result = mce_loop_f {
+    my ($mce, $slurp_ref, $chunk_id) = @_;
 
-      ## Quickly determine if a match is found.
-      ## Process slurped chunk only if true.
+    ## Quickly determine if a match is found.
+    ## Process slurped chunk only if true.
 
-      if ($$slurp_ref =~ /$pattern/m) {
-         my @matches;
+    if ($$slurp_ref =~ /$pattern/m) {
+       my @matches;
 
-         ## The following is fast on Unix. Performance degrades
-         ## drastically on Windows beyond 4 workers.
+       ## The following is fast on Unix. Performance degrades
+       ## drastically on Windows beyond 4 workers.
 
-         open my $MEM_FH, '<', $slurp_ref;
-         binmode $MEM_FH, ':raw';
-         while (<$MEM_FH>) { push @matches, $_ if (/$pattern/); }
-         close   $MEM_FH;
+       open my $MEM_FH, '<', $slurp_ref;
+       binmode $MEM_FH, ':raw';
+       while (<$MEM_FH>) { push @matches, $_ if (/$pattern/); }
+       close   $MEM_FH;
 
-         ## Therefore, use the following construct on Windows.
+       ## Therefore, use the following construct on Windows.
 
-         while ( $$slurp_ref =~ /([^\n]+\n)/mg ) {
-            my $line = $1; # save $1 to not lose the value
-            push @matches, $line if ($line =~ /$pattern/);
-         }
+       while ( $$slurp_ref =~ /([^\n]+\n)/mg ) {
+          my $line = $1; # save $1 to not lose the value
+          push @matches, $line if ($line =~ /$pattern/);
+       }
 
-         ## Gather matched lines.
+       ## Gather matched lines.
 
-         MCE->gather(@matches);
-      }
+       MCE->gather(@matches);
+    }
 
-   } $hugefile;
+ } $hugefile;
 
-   print join('', @result);
+ print join('', @result);
 
 =head1 OVERRIDING DEFAULTS
 
 The following list options which may be overridden when loading the module.
 
-   use Sereal qw( encode_sereal decode_sereal );
-   use CBOR::XS qw( encode_cbor decode_cbor );
-   use JSON::XS qw( encode_json decode_json );
+ use Sereal qw( encode_sereal decode_sereal );
+ use CBOR::XS qw( encode_cbor decode_cbor );
+ use JSON::XS qw( encode_json decode_json );
 
-   use MCE::Grep
-       max_workers => 4,                # Default 'auto'
-       chunk_size => 100,               # Default 'auto'
-       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
-       freeze => \&encode_sereal,       # \&Storable::freeze
-       thaw => \&decode_sereal          # \&Storable::thaw
-   ;
+ use MCE::Grep
+     max_workers => 4,                # Default 'auto'
+     chunk_size => 100,               # Default 'auto'
+     tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+     freeze => \&encode_sereal,       # \&Storable::freeze
+     thaw => \&decode_sereal          # \&Storable::thaw
+ ;
 
-There is a simpler way to enable Sereal. The following will attempt to use
-Sereal if available, otherwise defaults to Storable for serialization.
+From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
+Specify C<< Sereal => 0 >> to use Storable instead.
 
-   use MCE::Grep Sereal => 1;
-
-From MCE 1.800 onwards, this is done automatically if Sereal 3.008 or later
-is installed. Specify Sereal => 0 if Storable is desired.
+ use MCE::Grep Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -626,43 +611,105 @@ is installed. Specify Sereal => 0 if Storable is desired.
 
 =item MCE::Grep::init { options }
 
+=back
+
 The init function accepts a hash of MCE options. The gather option, if
 specified, is ignored due to being used internally by the module.
 
-   use MCE::Grep;
+ use MCE::Grep;
 
-   MCE::Grep::init {
-      chunk_size => 1, max_workers => 4,
+ MCE::Grep::init {
+    chunk_size => 1, max_workers => 4,
 
-      user_begin => sub {
-         print "## ", MCE->wid, " started\n";
-      },
+    user_begin => sub {
+       print "## ", MCE->wid, " started\n";
+    },
 
-      user_end => sub {
-         print "## ", MCE->wid, " completed\n";
-      }
-   };
+    user_end => sub {
+       print "## ", MCE->wid, " completed\n";
+    }
+ };
 
-   my @a = mce_grep { $_ % 5 == 0 } 1..100;
+ my @a = mce_grep { $_ % 5 == 0 } 1..100;
 
-   print "\n", "@a", "\n";
+ print "\n", "@a", "\n";
 
-   -- Output
+ -- Output
 
-   ## 2 started
-   ## 3 started
-   ## 1 started
-   ## 4 started
-   ## 3 completed
-   ## 4 completed
-   ## 1 completed
-   ## 2 completed
+ ## 2 started
+ ## 3 started
+ ## 1 started
+ ## 4 started
+ ## 3 completed
+ ## 4 completed
+ ## 1 completed
+ ## 2 completed
 
-   5 10 15 20 25 30 35 40 45 50 55 60 65 70 75 80 85 90 95 100
+ 5 10 15 20 25 30 35 40 45 50 55 60 65 70 75 80 85 90 95 100
+
+=head1 API DOCUMENTATION
+
+=over 3
+
+=item MCE::Grep->run ( sub { code }, list )
+
+=item mce_grep { code } list
 
 =back
 
-=head1 API DOCUMENTATION
+Input data may be defined using a list or an array reference. Unlike MCE::Loop,
+Flow, and Step, specifying a hash reference as input data isn't allowed.
+
+ ## Array or array_ref
+ my @a = mce_grep { /[2357]/ } 1..1000;
+ my @b = mce_grep { /[2357]/ } \@list;
+
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_grep { $_->[1] =~ /[2357]/ } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_grep { $_->[1] =~ /[2357]/ } \@deeply_list;
+
+ ## Not supported
+ my @z = mce_grep { ... } \%hash;
+
+=over 3
+
+=item MCE::Grep->run_file ( sub { code }, file )
+
+=item mce_grep_f { code } file
+
+=back
+
+The fastest of these is the /path/to/file. Workers communicate the next offset
+position among themselves with zero interaction by the manager process.
+
+C<IO::All> { File, Pipe, STDIO } is supported since MCE 1.845.
+
+ my @c = mce_grep_f { /pattern/ } "/path/to/file";  # faster
+ my @d = mce_grep_f { /pattern/ } $file_handle;
+ my @e = mce_grep_f { /pattern/ } $io;              # IO::All
+ my @f = mce_grep_f { /pattern/ } \$scalar;
+
+=over 3
+
+=item MCE::Grep->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_grep_s { code } $beg, $end [, $step, $fmt ]
+
+=back
+
+Sequence may be defined as a list, an array reference, or a hash reference.
+The functions require both begin and end values to run. Step and format are
+optional. The format is passed to sprintf (% may be omitted below).
+
+ my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
+
+ my @f = mce_grep_s { /[1234]\.[5678]/ } $beg, $end, $step, $fmt;
+ my @g = mce_grep_s { /[1234]\.[5678]/ } [ $beg, $end, $step, $fmt ];
+
+ my @h = mce_grep_s { /[1234]\.[5678]/ } {
+    begin => $beg, end => $end,
+    step => $step, format => $fmt
+ };
 
 =over 3
 
@@ -670,50 +717,13 @@ specified, is ignored due to being used internally by the module.
 
 =item mce_grep { code } iterator
 
-An iterator reference can by specified for input_data. Iterators are described
-under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   my @a = mce_grep { $_ % 3 == 0 } make_iterator(10, 30, 2);
-
-=item MCE::Grep->run ( sub { code }, list )
-
-=item mce_grep { code } list
-
-Input data can be defined using a list.
-
-   my @a = mce_grep { /[2357]/ } 1..1000;
-   my @b = mce_grep { /[2357]/ } [ 1..1000 ];
-
-=item MCE::Grep->run_file ( sub { code }, file )
-
-=item mce_grep_f { code } file
-
-The fastest of these is the /path/to/file. Workers communicate the next offset
-position among themselves without any interaction from the manager process.
-
-   my @c = mce_grep_f { /pattern/ } "/path/to/file";
-   my @d = mce_grep_f { /pattern/ } $file_handle;
-   my @e = mce_grep_f { /pattern/ } \$scalar;
-
-=item MCE::Grep->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
-
-=item mce_grep_s { code } $beg, $end [, $step, $fmt ]
-
-Sequence can be defined as a list, an array reference, or a hash reference.
-The functions require both begin and end values to run. Step and format are
-optional. The format is passed to sprintf (% may be omitted below).
-
-   my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
-
-   my @f = mce_grep_s { /[1234]\.[5678]/ } $beg, $end, $step, $fmt;
-   my @g = mce_grep_s { /[1234]\.[5678]/ } [ $beg, $end, $step, $fmt ];
-
-   my @h = mce_grep_s { /[1234]\.[5678]/ } {
-      begin => $beg, end => $end, step => $step, format => $fmt
-   };
-
 =back
 
+An iterator reference may be specified for input_data. Iterators are described
+under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
+
+ my @a = mce_grep { $_ % 3 == 0 } make_iterator(10, 30, 2);
+
 =head1 MANUAL SHUTDOWN
 
 =over 3
@@ -722,21 +732,21 @@ optional. The format is passed to sprintf (% may be omitted below).
 
 =item MCE::Grep::finish
 
+=back
+
 Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
 
-   use MCE::Grep;
+ use MCE::Grep;
 
-   MCE::Grep::init {
-      chunk_size => 20, max_workers => 'auto'
-   };
+ MCE::Grep::init {
+    chunk_size => 20, max_workers => 'auto'
+ };
 
-   my @a = mce_grep { ... } 1..100;
+ my @a = mce_grep { ... } 1..100;
 
-   MCE::Grep::finish;
-
-=back
+ MCE::Grep::finish;
 
 =head1 INDEX
 
