@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Parallel loop model for building creative loops.
+## MCE model for building parallel loops.
 ##
 ###############################################################################
 
@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -22,12 +22,10 @@ use MCE;
 
 our @CARP_NOT = qw( MCE );
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
-my $_caller;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ###############################################################################
@@ -63,7 +61,7 @@ sub import {
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
 
-      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
          if ( shift eq '0' ) {
             require Storable;
@@ -76,19 +74,15 @@ sub import {
       _croak("Error: ($_argument) invalid module option");
    }
 
-   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+   $_p->{MAX_WORKERS} = MCE::_parse_max_workers($_p->{MAX_WORKERS});
 
-   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
-   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+   MCE::_validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS', $_tag);
+   MCE::_validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE', $_tag)
       unless ($_p->{CHUNK_SIZE} eq 'auto');
 
    return;
 }
 
-END {
-   %{ $_MCE } = ();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Init and finish routines.
@@ -98,7 +92,7 @@ END {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
-   MCE::Loop->finish( my $_pkg = "$$.$_tid.".caller() );
+   my $_pkg = "$$.$_tid.".caller();
 
    $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
@@ -112,17 +106,18 @@ sub finish (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
    my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
 
-   if ( $_pkg eq 'MCE::Shared::Server' ) {
-      MCE::Loop->finish($_) for ( keys %{ $_MCE } );
-      %{ $_MCE } = ();
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Loop->finish($_k, 1); }
    }
-   elsif ( exists $_MCE->{$_pkg} ) {
-      MCE::_save_state(), $_MCE->{$_pkg}->shutdown(@_), MCE::_restore_state()
-         if $_MCE->{$_pkg}{_spawned};
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
 
       delete $_prev_c->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
+   @_ = ();
+
    return;
 }
 
@@ -137,7 +132,7 @@ sub run_file (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift; my $_file = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -153,7 +148,7 @@ sub run_file (&@) {
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
       $_params->{$_pid}{_file} = $_file;
    }
    else {
@@ -176,7 +171,7 @@ sub run_seq (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -196,7 +191,7 @@ sub run_seq (&@) {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
       $_params->{$_pid}{sequence} = $_[0];
    }
-   elsif (ref $_[0] eq '') {
+   elsif (ref $_[0] eq '' || ref($_[0]) =~ /^Math::/) {
       $_begin = $_[0]; $_end = $_[1];
       $_params->{$_pid}{sequence} = [ @_ ];
    }
@@ -209,7 +204,7 @@ sub run_seq (&@) {
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{$_pid}{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = undef;
 
    @_ = ();
 
@@ -227,18 +222,18 @@ sub run (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift;
-   my $_pkg  = defined $_caller ? $_caller : caller();  $_caller = undef;
+   my $_pkg  = caller() eq 'MCE::Loop' ? caller(1) : caller();
    my $_pid  = "$$.$_tid.$_pkg";
 
    my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
    my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+   if (@_ == 1 && $_r =~ /^(?:ARRAY|HASH|SCALAR|CODE|GLOB|FileHandle|IO::)/) {
       $_input_data = shift;
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
-      $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
+      $_max_workers = MCE::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
       delete $_p->{sequence}   if (defined $_input_data || scalar @_);
@@ -246,7 +241,7 @@ sub run (&@) {
       delete $_p->{user_tasks} if (exists $_p->{user_tasks});
    }
 
-   my $_chunk_size = MCE::Util::_parse_chunk_size(
+   my $_chunk_size = MCE::_parse_chunk_size(
       $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
       $_input_data, scalar @_
    );
@@ -261,7 +256,7 @@ sub run (&@) {
 
    ## -------------------------------------------------------------------------
 
-   MCE::_save_state();
+   MCE::_save_state($_MCE->{$_pid});
 
    if (!defined $_prev_c->{$_pid} || $_prev_c->{$_pid} != $_code) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
@@ -278,7 +273,7 @@ sub run (&@) {
             next if ($_k eq 'input_data');
             next if ($_k eq 'chunk_size');
 
-            _croak("MCE::Loop: ($_k) is not a valid constructor argument")
+            _croak("$_tag: ($_k) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_k});
 
             $_opts{$_k} = $_p->{$_k};
@@ -320,14 +315,10 @@ sub run (&@) {
       }
    }
 
-   delete $_MCE->{$_pid}{gather} if (defined $_wa);
-
-   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
-      $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
-   }
-
    MCE::_restore_state();
 
+   delete $_MCE->{$_pid}{gather} if (defined $_wa);
+
    return ((defined $_wa) ? @_a : ());
 }
 
@@ -342,21 +333,6 @@ sub _croak {
    goto &MCE::_croak;
 }
 
-sub _validate_number {
-
-   my ($_n, $_key) = @_;
-
-   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
-
-   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
-
-   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
-      _croak("$_tag: ($_key) is not valid");
-   }
-
-   return;
-}
-
 1;
 
 __END__
@@ -369,11 +345,11 @@ __END__
 
 =head1 NAME
 
-MCE::Loop - Parallel loop model for building creative loops
+MCE::Loop - MCE model for building parallel loops
 
 =head1 VERSION
 
-This document describes MCE::Loop version 1.799_03
+This document describes MCE::Loop version 1.867_01
 
 =head1 DESCRIPTION
 
@@ -382,57 +358,57 @@ MCE::Loop is not MCE::Map but more along the lines of an easy way to spin up a
 MCE instance and have user_func pointing to your code block. If you want
 something similar to map, then see L<MCE::Map>.
 
-   ## Construction when chunking is not desired
+ ## Construction when chunking is not desired
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      max_workers => 5, chunk_size => 1
-   };
+ MCE::Loop::init {
+    max_workers => 5, chunk_size => 1
+ };
 
-   mce_loop {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
-      MCE->say("$chunk_id: $_");
-   } 40 .. 48;
+ mce_loop {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    MCE->say("$chunk_id: $_");
+ } 40 .. 48;
 
-   -- Output
+ -- Output
 
-   3: 42
-   1: 40
-   2: 41
-   4: 43
-   5: 44
-   6: 45
-   7: 46
-   8: 47
-   9: 48
+ 3: 42
+ 1: 40
+ 2: 41
+ 4: 43
+ 5: 44
+ 6: 45
+ 7: 46
+ 8: 47
+ 9: 48
 
-   ## Construction for 'auto' or greater than 1
+ ## Construction for 'auto' or greater than 1
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      max_workers => 5, chunk_size => 'auto'
-   };
+ MCE::Loop::init {
+    max_workers => 5, chunk_size => 'auto'
+ };
 
-   mce_loop {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
-      for (@{ $chunk_ref }) {
-         MCE->say("$chunk_id: $_");
-      }
-   } 40 .. 48;
+ mce_loop {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    for (@{ $chunk_ref }) {
+       MCE->say("$chunk_id: $_");
+    }
+ } 40 .. 48;
 
-   -- Output
+ -- Output
 
-   1: 40
-   2: 42
-   1: 41
-   4: 46
-   2: 43
-   5: 48
-   3: 44
-   4: 47
-   3: 45
+ 1: 40
+ 2: 42
+ 1: 41
+ 4: 46
+ 2: 43
+ 5: 48
+ 3: 44
+ 4: 47
+ 3: 45
 
 =head1 SYNOPSIS when CHUNK_SIZE EQUALS 1
 
@@ -444,70 +420,83 @@ variable $_ when chunk_size equals 1. Otherwise, $_ points to $chunk_ref
 containing many items. Basically, line 2 below may be omitted from your code
 when using $_. One can call MCE->chunk_id to obtain the current chunk id.
 
-   line 1:  user_func => sub {
-   line 2:     my ($mce, $chunk_ref, $chunk_id) = @_;
-   line 3:
-   line 4:     $_ points to $chunk_ref->[0]
-   line 5:        in MCE 1.5 when chunk_size == 1
-   line 6:
-   line 7:     $_ points to $chunk_ref
-   line 8:        in MCE 1.5 when chunk_size  > 1
-   line 9:  }
+ line 1:  user_func => sub {
+ line 2:     my ($mce, $chunk_ref, $chunk_id) = @_;
+ line 3:
+ line 4:     $_ points to $chunk_ref->[0]
+ line 5:        in MCE 1.5 when chunk_size == 1
+ line 6:
+ line 7:     $_ points to $chunk_ref
+ line 8:        in MCE 1.5 when chunk_size  > 1
+ line 9:  }
 
 Follow this synopsis when chunk_size equals one. Looping is not required from
 inside the block. Hence, the block is called once per each item.
 
-   ## Exports mce_loop, mce_loop_f, and mce_loop_s
-   use MCE::Loop;
+ ## Exports mce_loop, mce_loop_f, and mce_loop_s
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      chunk_size => 1
-   };
+ MCE::Loop::init {
+    chunk_size => 1
+ };
 
-   ## Array or array_ref
-   mce_loop { do_work($_) } 1..10000;
-   mce_loop { do_work($_) } [ 1..10000 ];
+ ## Array or array_ref
+ mce_loop { do_work($_) } 1..10000;
+ mce_loop { do_work($_) } \@list;
 
-   ## File_path, glob_ref, or scalar_ref
-   mce_loop_f { chomp; do_work($_) } "/path/to/file";
-   mce_loop_f { chomp; do_work($_) } $file_handle;
-   mce_loop_f { chomp; do_work($_) } \$scalar;
+ ## Important; pass an array_ref for deeply input data
+ mce_loop { do_work($_) } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ mce_loop { do_work($_) } \@deeply_list;
 
-   ## Sequence of numbers (begin, end [, step, format])
-   mce_loop_s { do_work($_) } 1, 10000, 5;
-   mce_loop_s { do_work($_) } [ 1, 10000, 5 ];
+ ## File path, glob ref, IO::All::{ File, Pipe, STDIO } obj, or scalar ref
+ ## Workers read directly and not involve the manager process
+ mce_loop_f { chomp; do_work($_) } "/path/to/file"; # efficient
 
-   mce_loop_s { do_work($_) } {
-      begin => 1, end => 10000, step => 5, format => undef
-   };
+ ## Involves the manager process, therefore slower
+ mce_loop_f { chomp; do_work($_) } $file_handle;
+ mce_loop_f { chomp; do_work($_) } $io;
+ mce_loop_f { chomp; do_work($_) } \$scalar;
+
+ ## Sequence of numbers (begin, end [, step, format])
+ mce_loop_s { do_work($_) } 1, 10000, 5;
+ mce_loop_s { do_work($_) } [ 1, 10000, 5 ];
+
+ mce_loop_s { do_work($_) } {
+    begin => 1, end => 10000, step => 5, format => undef
+ };
 
 =head1 SYNOPSIS when CHUNK_SIZE is GREATER THAN 1
 
 Follow this synopsis when chunk_size equals 'auto' or greater than 1.
 This means having to loop through the chunk from inside the block.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {          ## Chunk_size defaults to 'auto' when
-      chunk_size => 'auto'    ## not specified. Therefore, the init
-   };                         ## function may be omitted.
+ MCE::Loop::init {          ## Chunk_size defaults to 'auto' when
+    chunk_size => 'auto'    ## not specified. Therefore, the init
+ };                         ## function may be omitted.
 
-   ## Syntax is shown for mce_loop for demonstration purposes.
-   ## Looping inside the block is the same for mce_loop_f and
-   ## mce_loop_s.
+ ## Syntax is shown for mce_loop for demonstration purposes.
+ ## Looping inside the block is the same for mce_loop_f and
+ ## mce_loop_s.
 
-   mce_loop { do_work($_) for (@{ $_ }) } 1..10000;
+ ## Array or array_ref
+ mce_loop { do_work($_) for (@{ $_ }) } 1..10000;
+ mce_loop { do_work($_) for (@{ $_ }) } \@list;
 
-   ## Same as above, resembles code using the Core API.
+ ## Important; pass an array_ref for deeply input data
+ mce_loop { do_work($_) for (@{ $_ }) } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ mce_loop { do_work($_) for (@{ $_ }) } \@deeply_list;
 
-   mce_loop {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
+ ## Resembles code using the core MCE API
+ mce_loop {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      for (@{ $chunk_ref }) {
-         do_work($_);
-      }
+    for (@{ $chunk_ref }) {
+       do_work($_);
+    }
 
-   } 1..10000;
+ } 1..10000;
 
 Chunking reduces the number of IPC calls behind the scene. Think in terms of
 chunks whenever processing a large amount of data. For relatively small data,
@@ -517,25 +506,22 @@ choosing 1 for chunk_size is fine.
 
 The following list options which may be overridden when loading the module.
 
-   use Sereal qw( encode_sereal decode_sereal );
-   use CBOR::XS qw( encode_cbor decode_cbor );
-   use JSON::XS qw( encode_json decode_json );
+ use Sereal qw( encode_sereal decode_sereal );
+ use CBOR::XS qw( encode_cbor decode_cbor );
+ use JSON::XS qw( encode_json decode_json );
 
-   use MCE::Loop
-       max_workers => 4,                # Default 'auto'
-       chunk_size => 100,               # Default 'auto'
-       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
-       freeze => \&encode_sereal,       # \&Storable::freeze
-       thaw => \&decode_sereal          # \&Storable::thaw
-   ;
+ use MCE::Loop
+     max_workers => 4,                # Default 'auto'
+     chunk_size => 100,               # Default 'auto'
+     tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+     freeze => \&encode_sereal,       # \&Storable::freeze
+     thaw => \&decode_sereal          # \&Storable::thaw
+ ;
 
-There is a simpler way to enable Sereal. The following will attempt to use
-Sereal if available, otherwise defaults to Storable for serialization.
+From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
+Specify C<< Sereal => 0 >> to use Storable instead.
 
-   use MCE::Loop Sereal => 1;
-
-From MCE 1.800 onwards, this is done automatically if Sereal 3.008 or later
-is installed. Specify Sereal => 0 if Storable is desired.
+ use MCE::Loop Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -545,100 +531,164 @@ is installed. Specify Sereal => 0 if Storable is desired.
 
 =item MCE::Loop::init { options }
 
+=back
+
 The init function accepts a hash of MCE options.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      chunk_size => 1, max_workers => 4,
+ MCE::Loop::init {
+    chunk_size => 1, max_workers => 4,
 
-      user_begin => sub {
-         print "## ", MCE->wid, " started\n";
-      },
+    user_begin => sub {
+       print "## ", MCE->wid, " started\n";
+    },
 
-      user_end => sub {
-         print "## ", MCE->wid, " completed\n";
-      }
-   };
+    user_end => sub {
+       print "## ", MCE->wid, " completed\n";
+    }
+ };
 
-   my %a = mce_loop { MCE->gather($_, $_ * $_) } 1..100;
+ my %a = mce_loop { MCE->gather($_, $_ * $_) } 1..100;
 
-   print "\n", "@a{1..100}", "\n";
+ print "\n", "@a{1..100}", "\n";
 
-   -- Output
+ -- Output
 
-   ## 3 started
-   ## 1 started
-   ## 2 started
-   ## 4 started
-   ## 1 completed
-   ## 2 completed
-   ## 3 completed
-   ## 4 completed
+ ## 3 started
+ ## 1 started
+ ## 2 started
+ ## 4 started
+ ## 1 completed
+ ## 2 completed
+ ## 3 completed
+ ## 4 completed
 
-   1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
-   400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
-   1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
-   2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
-   3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
-   5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
-   7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
-   10000
-
-=back
+ 1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
+ 400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
+ 1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
+ 2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
+ 3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
+ 5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
+ 7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
+ 10000
 
 =head1 API DOCUMENTATION
 
 The following assumes chunk_size equals 1 in order to demonstrate all the
-possibilities of passing input data into the code block.
+possibilities for providing input data.
 
 =over 3
 
-=item MCE::Loop->run ( sub { code }, iterator )
-
-=item mce_loop { code } iterator
-
-An iterator reference can by specified for input_data. Iterators are described
-under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   mce_loop { $_ } make_iterator(10, 30, 2);
-
 =item MCE::Loop->run ( sub { code }, list )
 
 =item mce_loop { code } list
 
-Input data can be defined using a list.
+=back
 
-   mce_loop { $_ } 1..1000;
-   mce_loop { $_ } [ 1..1000 ];
+Input data may be defined using a list, an array ref, or a hash ref.
+
+ # $_ contains the item when chunk_size => 1
+
+ mce_loop { do_work($_) } 1..1000;
+ mce_loop { do_work($_) } \@list;
+
+ # Important; pass an array_ref for deeply input data
+
+ mce_loop { do_work($_) } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ mce_loop { do_work($_) } \@deeply_list;
+
+ # Chunking; any chunk_size => 1 or greater
+
+ my %res = mce_loop {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my %ret;
+    for my $item (@{ $chunk_ref }) {
+       $ret{$item} = $item * 2;
+    }
+    MCE->gather(%ret);
+ }
+ \@list;
+
+ # Input hash; current API available since 1.828
+
+ my %res = mce_loop {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my %ret;
+    for my $key (keys %{ $chunk_ref }) {
+       $ret{$key} = $chunk_ref->{$key} * 2;
+    }
+    MCE->gather(%ret);
+ }
+ \%hash;
+
+=over 3
 
 =item MCE::Loop->run_file ( sub { code }, file )
 
 =item mce_loop_f { code } file
 
-The fastest of these is the /path/to/file. Workers communicate the next offset
-position among themselves without any interaction from the manager process.
+=back
 
-   mce_loop_f { $_ } "/path/to/file";
-   mce_loop_f { $_ } $file_handle;
-   mce_loop_f { $_ } \$scalar;
+The fastest of these is the /path/to/file. Workers communicate the next offset
+position among themselves with zero interaction by the manager process.
+
+C<IO::All> { File, Pipe, STDIO } is supported since MCE 1.845.
+
+ # $_ contains the line when chunk_size => 1
+
+ mce_loop_f { $_ } "/path/to/file";  # faster
+ mce_loop_f { $_ } $file_handle;
+ mce_loop_f { $_ } $io;              # IO::All
+ mce_loop_f { $_ } \$scalar;
+
+ # chunking, any chunk_size => 1 or greater
+
+ my %res = mce_loop_f {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my $buf = '';
+    for my $line (@{ $chunk_ref }) {
+       $buf .= $line;
+    }
+    MCE->gather($chunk_id, $buf);
+ }
+ "/path/to/file";
+
+=over 3
 
 =item MCE::Loop->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
 
 =item mce_loop_s { code } $beg, $end [, $step, $fmt ]
 
-Sequence can be defined as a list, an array reference, or a hash reference.
+=back
+
+Sequence may be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
 optional. The format is passed to sprintf (% may be omitted below).
 
-   my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
+ my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
 
-   mce_loop_s { $_ } $beg, $end, $step, $fmt;
-   mce_loop_s { $_ } [ $beg, $end, $step, $fmt ];
+ # $_ contains the sequence number when chunk_size => 1
 
-   mce_loop_s { $_ } {
-      begin => $beg, end => $end, step => $step, format => $fmt
-   };
+ mce_loop_s { $_ } $beg, $end, $step, $fmt;
+ mce_loop_s { $_ } [ $beg, $end, $step, $fmt ];
+
+ mce_loop_s { $_ } {
+    begin => $beg, end => $end,
+    step => $step, format => $fmt
+ };
+
+ # chunking, any chunk_size => 1 or greater
+
+ my %res = mce_loop_s {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my $buf = '';
+    for my $seq (@{ $chunk_ref }) {
+       $buf .= "$seq\n";
+    }
+    MCE->gather($chunk_id, $buf);
+ }
+ [ $beg, $end ];
 
 The sequence engine can compute 'begin' and 'end' items only, for the chunk,
 and not the items in between (hence boundaries only). This option applies
@@ -651,183 +701,192 @@ for looping inside the block; e.g. Monte Carlo simulations.
 
 Time was measured using 1 worker to emphasize the difference.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      max_workers => 1, chunk_size => 1_250_000,
-      bounds_only => 1
-   };
+ MCE::Loop::init {
+    max_workers => 1, chunk_size => 1_250_000,
+    bounds_only => 1
+ };
 
-   ## For sequence, the input scalar $_ points to $chunk_ref
-   ## when chunk_size > 1, otherwise $chunk_ref->[0].
-   ##
-   ## mce_loop_s {
-   ##    my $begin = $_->[0]; my $end = $_->[-1];
-   ##
-   ##    for ($begin .. $end) {
-   ##       ...
-   ##    }
-   ##
-   ## } 1, 10_000_000;
+ # Typically, the input scalar $_ contains the sequence number
+ # when chunk_size => 1, unless the bounds_only option is set
+ # which is the case here. Thus, $_ points to $chunk_ref.
 
-   mce_loop_s {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
-      ## $chunk_ref contains 2 items, not 1_250_000
+ mce_loop_s {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      my $begin = $chunk_ref->[ 0];
-      my $end   = $chunk_ref->[-1];   ## or $chunk_ref->[1]
+    # $chunk_ref contains 2 items, not 1_250_000
+    # my ( $begin, $end ) = ( $_->[0], $_->[1] );
 
-      MCE->printf("%7d .. %8d\n", $begin, $end);
+    my $begin = $chunk_ref->[0];
+    my $end   = $chunk_ref->[1];
 
-   } 1, 10_000_000;
+    # for my $seq ( $begin .. $end ) {
+    #    ...
+    # }
 
-   -- Output
+    MCE->printf("%7d .. %8d\n", $begin, $end);
+ }
+ [ 1, 10_000_000 ];
 
-         1 ..  1250000
-   1250001 ..  2500000
-   2500001 ..  3750000
-   3750001 ..  5000000
-   5000001 ..  6250000
-   6250001 ..  7500000
-   7500001 ..  8750000
-   8750001 .. 10000000
+ -- Output
+
+       1 ..  1250000
+ 1250001 ..  2500000
+ 2500001 ..  3750000
+ 3750001 ..  5000000
+ 5000001 ..  6250000
+ 6250001 ..  7500000
+ 7500001 ..  8750000
+ 8750001 .. 10000000
+
+=over 3
+
+=item MCE::Loop->run ( sub { code }, iterator )
+
+=item mce_loop { code } iterator
 
 =back
 
+An iterator reference may be specified for input_data. Iterators are described
+under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
+
+ mce_loop { $_ } make_iterator(10, 30, 2);
+
 =head1 GATHERING DATA
 
 Unlike MCE::Map where gather and output order are done for you automatically,
 the gather method is used to have results sent back to the manager process.
 
-   use MCE::Loop chunk_size => 1;
+ use MCE::Loop chunk_size => 1;
 
-   ## Output order is not guaranteed.
-   my @a = mce_loop { MCE->gather($_ * 2) } 1..100;
-   print "@a\n\n";
+ ## Output order is not guaranteed.
+ my @a1 = mce_loop { MCE->gather($_ * 2) } 1..100;
+ print "@a1\n\n";
 
-   ## Outputs to a hash instead (key, value).
-   my %h1 = mce_loop { MCE->gather($_, $_ * 2) } 1..100;
-   print "@h1{1..100}\n\n";
+ ## Outputs to a hash instead (key, value).
+ my %h1 = mce_loop { MCE->gather($_, $_ * 2) } 1..100;
+ print "@h1{1..100}\n\n";
 
-   ## This does the same thing due to chunk_id starting at one.
-   my %h2 = mce_loop { MCE->gather(MCE->chunk_id, $_ * 2) } 1..100;
-   print "@h2{1..100}\n\n";
+ ## This does the same thing due to chunk_id starting at one.
+ my %h2 = mce_loop { MCE->gather(MCE->chunk_id, $_ * 2) } 1..100;
+ print "@h2{1..100}\n\n";
 
-The gather method can be called multiple times within the block unlike return
+The gather method may be called multiple times within the block unlike return
 which would leave the block. Therefore, think of gather as yielding results
 immediately to the manager process without actually leaving the block.
 
-   use MCE::Loop chunk_size => 1, max_workers => 3;
+ use MCE::Loop chunk_size => 1, max_workers => 3;
 
-   my @hosts = qw(
-      hosta hostb hostc hostd hoste
-   );
+ my @hosts = qw(
+    hosta hostb hostc hostd hoste
+ );
 
-   my %h3 = mce_loop {
-      my ($output, $error, $status); my $host = $_;
+ my %h3 = mce_loop {
+    my ($output, $error, $status); my $host = $_;
 
-      ## Do something with $host;
-      $output = "Worker ". MCE->wid .": Hello from $host";
+    ## Do something with $host;
+    $output = "Worker ". MCE->wid .": Hello from $host";
 
-      if (MCE->chunk_id % 3 == 0) {
-         ## Simulating an error condition
-         local $? = 1; $status = $?;
-         $error = "Error from $host"
-      }
-      else {
-         $status = 0;
-      }
+    if (MCE->chunk_id % 3 == 0) {
+       ## Simulating an error condition
+       local $? = 1; $status = $?;
+       $error = "Error from $host"
+    }
+    else {
+       $status = 0;
+    }
 
-      ## Ensure unique keys (key, value) when gathering to
-      ## a hash.
-      MCE->gather("$host.out", $output);
-      MCE->gather("$host.err", $error) if (defined $error);
-      MCE->gather("$host.sta", $status);
+    ## Ensure unique keys (key, value) when gathering to
+    ## a hash.
+    MCE->gather("$host.out", $output);
+    MCE->gather("$host.err", $error) if (defined $error);
+    MCE->gather("$host.sta", $status);
 
-   } @hosts;
+ } @hosts;
 
-   foreach my $host (@hosts) {
-      print $h3{"$host.out"}, "\n";
-      print $h3{"$host.err"}, "\n" if (exists $h3{"$host.err"});
-      print "Exit status: ", $h3{"$host.sta"}, "\n\n";
-   }
+ foreach my $host (@hosts) {
+    print $h3{"$host.out"}, "\n";
+    print $h3{"$host.err"}, "\n" if (exists $h3{"$host.err"});
+    print "Exit status: ", $h3{"$host.sta"}, "\n\n";
+ }
 
-   -- Output
+ -- Output
 
-   Worker 2: Hello from hosta
-   Exit status: 0
+ Worker 2: Hello from hosta
+ Exit status: 0
 
-   Worker 1: Hello from hostb
-   Exit status: 0
+ Worker 1: Hello from hostb
+ Exit status: 0
 
-   Worker 3: Hello from hostc
-   Error from hostc
-   Exit status: 1
+ Worker 3: Hello from hostc
+ Error from hostc
+ Exit status: 1
 
-   Worker 2: Hello from hostd
-   Exit status: 0
+ Worker 2: Hello from hostd
+ Exit status: 0
 
-   Worker 1: Hello from hoste
-   Exit status: 0
+ Worker 1: Hello from hoste
+ Exit status: 0
 
 The following uses an anonymous array containing 3 elements when gathering
 data. Serialization is automatic behind the scene.
 
-   my %h3 = mce_loop {
-      ...
+ my %h3 = mce_loop {
+    ...
 
-      MCE->gather($host, [$output, $error, $status]);
+    MCE->gather($host, [$output, $error, $status]);
 
-   } @hosts;
+ } @hosts;
 
-   foreach my $host (@hosts) {
-      print $h3{$host}->[0], "\n";
-      print $h3{$host}->[1], "\n" if (defined $h3{$host}->[1]);
-      print "Exit status: ", $h3{$host}->[2], "\n\n";
-   }
+ foreach my $host (@hosts) {
+    print $h3{$host}->[0], "\n";
+    print $h3{$host}->[1], "\n" if (defined $h3{$host}->[1]);
+    print "Exit status: ", $h3{$host}->[2], "\n\n";
+ }
 
 Although MCE::Map comes to mind, one may want additional control when
 gathering data such as retaining output order.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   sub preserve_order {
-      my %tmp; my $order_id = 1; my $gather_ref = $_[0];
+ sub preserve_order {
+    my %tmp; my $order_id = 1; my $gather_ref = $_[0];
 
-      return sub {
-         $tmp{ (shift) } = \@_;
+    return sub {
+       $tmp{ (shift) } = \@_;
 
-         while (1) {
-            last unless exists $tmp{$order_id};
-            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
-         }
+       while (1) {
+          last unless exists $tmp{$order_id};
+          push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
+       }
 
-         return;
-      };
-   }
+       return;
+    };
+ }
 
-   my @m2;
+ my @m2;
 
-   MCE::Loop::init {
-      chunk_size => 'auto', max_workers => 'auto',
-      gather => preserve_order(\@m2)
-   };
+ MCE::Loop::init {
+    chunk_size => 'auto', max_workers => 'auto',
+    gather => preserve_order(\@m2)
+ };
 
-   mce_loop {
-      my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
+ mce_loop {
+    my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      ## Compute the entire chunk data at once.
-      push @a, map { $_ * 2 } @{ $chunk_ref };
+    ## Compute the entire chunk data at once.
+    push @a, map { $_ * 2 } @{ $chunk_ref };
 
-      ## Afterwards, invoke the gather feature, which
-      ## will direct the data to the callback function.
-      MCE->gather(MCE->chunk_id, @a);
+    ## Afterwards, invoke the gather feature, which
+    ## will direct the data to the callback function.
+    MCE->gather(MCE->chunk_id, @a);
 
-   } 1..100000;
+ } 1..100000;
 
-   MCE::Loop::finish;
+ MCE::Loop::finish;
 
-   print scalar @m2, "\n";
+ print scalar @m2, "\n";
 
 All 6 models support 'auto' for chunk_size unlike the Core API. Think of the
 models as the basis for providing JIT for MCE. They create the instance, tune
@@ -835,33 +894,33 @@ max_workers, and tune chunk_size automatically regardless of the hardware.
 
 The following does the same thing using the Core API.
 
-   use MCE;
+ use MCE;
 
-   sub preserve_order {
-      ...
-   }
+ sub preserve_order {
+    ...
+ }
 
-   my $mce = MCE->new(
-      max_workers => 'auto', chunk_size => 8000,
+ my $mce = MCE->new(
+    max_workers => 'auto', chunk_size => 8000,
 
-      user_func => sub {
-         my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
+    user_func => sub {
+       my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-         ## Compute the entire chunk data at once.
-         push @a, map { $_ * 2 } @{ $chunk_ref };
+       ## Compute the entire chunk data at once.
+       push @a, map { $_ * 2 } @{ $chunk_ref };
 
-         ## Afterwards, invoke the gather feature, which
-         ## will direct the data to the callback function.
-         MCE->gather(MCE->chunk_id, @a);
-      }
-   );
+       ## Afterwards, invoke the gather feature, which
+       ## will direct the data to the callback function.
+       MCE->gather(MCE->chunk_id, @a);
+    }
+ );
 
-   my @m2;
+ my @m2;
 
-   $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
-   $mce->shutdown;
+ $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
+ $mce->shutdown;
 
-   print scalar @m2, "\n";
+ print scalar @m2, "\n";
 
 =head1 MANUAL SHUTDOWN
 
@@ -871,21 +930,21 @@ The following does the same thing using the Core API.
 
 =item MCE::Loop::finish
 
+=back
+
 Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
 
-   use MCE::Loop;
+ use MCE::Loop;
 
-   MCE::Loop::init {
-      chunk_size => 20, max_workers => 'auto'
-   };
+ MCE::Loop::init {
+    chunk_size => 20, max_workers => 'auto'
+ };
 
-   mce_loop { ... } 1..100;
+ mce_loop { ... } 1..100;
 
-   MCE::Loop::finish;
-
-=back
+ MCE::Loop::finish;
 
 =head1 INDEX
 
