@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -22,12 +22,10 @@ use MCE;
 
 our @CARP_NOT = qw( MCE );
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
-my $_caller;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ###############################################################################
@@ -63,7 +61,7 @@ sub import {
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
 
-      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
          if ( shift eq '0' ) {
             require Storable;
@@ -76,19 +74,15 @@ sub import {
       _croak("Error: ($_argument) invalid module option");
    }
 
-   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+   $_p->{MAX_WORKERS} = MCE::_parse_max_workers($_p->{MAX_WORKERS});
 
-   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
-   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+   MCE::_validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS', $_tag);
+   MCE::_validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE', $_tag)
       unless ($_p->{CHUNK_SIZE} eq 'auto');
 
    return;
 }
 
-END {
-   %{ $_MCE } = ();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Gather callback for storing by chunk_id => chunk_ref into a hash.
@@ -116,10 +110,13 @@ sub _gather {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Map');
-   MCE::Map->finish( my $_pkg = "$$.$_tid.".caller() );
+   my $_pkg = "$$.$_tid.".caller();
 
    $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
+   _croak("$_tag: (HASH) not allowed as input by this MCE model")
+      if ( ref $_params->{$_pkg}{input_data} eq 'HASH' );
+
    @_ = ();
 
    return;
@@ -130,19 +127,19 @@ sub finish (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Map');
    my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
 
-   if ( $_pkg eq 'MCE::Shared::Server' ) {
-      MCE::Map->finish($_) for ( keys %{ $_MCE } );
-      %{ $_MCE } = ();
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Map->finish($_k, 1); }
    }
-   elsif ( exists $_MCE->{$_pkg} ) {
-      MCE::_save_state(), $_MCE->{$_pkg}->shutdown(@_), MCE::_restore_state()
-         if $_MCE->{$_pkg}{_spawned};
-
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
       $_total_chunks = undef, undef %_tmp;
 
       delete $_prev_c->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
+   @_ = ();
+
    return;
 }
 
@@ -157,7 +154,7 @@ sub run_file (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Map');
 
    my $_code = shift; my $_file = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -173,7 +170,7 @@ sub run_file (&@) {
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
       $_params->{$_pid}{_file} = $_file;
    }
    else {
@@ -196,7 +193,7 @@ sub run_seq (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Map');
 
    my $_code = shift;
-   my $_pid  = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid  = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -216,7 +213,7 @@ sub run_seq (&@) {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
       $_params->{$_pid}{sequence} = $_[0];
    }
-   elsif (ref $_[0] eq '') {
+   elsif (ref $_[0] eq '' || ref($_[0]) =~ /^Math::/) {
       $_begin = $_[0]; $_end = $_[1];
       $_params->{$_pid}{sequence} = [ @_ ];
    }
@@ -229,7 +226,7 @@ sub run_seq (&@) {
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{$_pid}{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = undef;
 
    @_ = ();
 
@@ -247,18 +244,20 @@ sub run (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Map');
 
    my $_code = shift;  $_total_chunks = 0; undef %_tmp;
-   my $_pkg  = defined $_caller ? $_caller : caller();  $_caller = undef;
+   my $_pkg  = caller() eq 'MCE::Map' ? caller(1) : caller();
    my $_pid  = "$$.$_tid.$_pkg";
 
    my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
    my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+   if (@_ == 1 && $_r =~ /^(?:ARRAY|HASH|SCALAR|CODE|GLOB|FileHandle|IO::)/) {
+      _croak("$_tag: (HASH) not allowed as input by this MCE model")
+         if $_r eq 'HASH';
       $_input_data = shift;
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
-      $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
+      $_max_workers = MCE::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
       delete $_p->{sequence}    if (defined $_input_data || scalar @_);
@@ -269,7 +268,7 @@ sub run (&@) {
       delete $_p->{gather}      if (exists $_p->{gather});
    }
 
-   my $_chunk_size = MCE::Util::_parse_chunk_size(
+   my $_chunk_size = MCE::_parse_chunk_size(
       $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
       $_input_data, scalar @_
    );
@@ -284,7 +283,7 @@ sub run (&@) {
 
    ## -------------------------------------------------------------------------
 
-   MCE::_save_state();
+   MCE::_save_state($_MCE->{$_pid});
 
    if (!defined $_prev_c->{$_pid} || $_prev_c->{$_pid} != $_code) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
@@ -348,7 +347,7 @@ sub run (&@) {
             next if ($_k eq 'input_data');
             next if ($_k eq 'chunk_size');
 
-            _croak("MCE::Map: ($_k) is not a valid constructor argument")
+            _croak("$_tag: ($_k) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_k});
 
             $_opts{$_k} = $_p->{$_k};
@@ -396,10 +395,6 @@ sub run (&@) {
       }
    }
 
-   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
-      $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
-   }
-
    MCE::_restore_state();
 
    if ($_wantarray) {
@@ -423,21 +418,6 @@ sub _croak {
    goto &MCE::_croak;
 }
 
-sub _validate_number {
-
-   my ($_n, $_key) = @_;
-
-   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
-
-   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
-
-   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
-      _croak("$_tag: ($_key) is not valid");
-   }
-
-   return;
-}
-
 1;
 
 __END__
@@ -454,29 +434,37 @@ MCE::Map - Parallel map model similar to the native map function
 
 =head1 VERSION
 
-This document describes MCE::Map version 1.799_03
+This document describes MCE::Map version 1.867_01
 
 =head1 SYNOPSIS
 
-   ## Exports mce_map, mce_map_f, and mce_map_s
-   use MCE::Map;
+ ## Exports mce_map, mce_map_f, and mce_map_s
+ use MCE::Map;
 
-   ## Array or array_ref
-   my @a = mce_map { $_ * $_ } 1..10000;
-   my @b = mce_map { $_ * $_ } [ 1..10000 ];
+ ## Array or array_ref
+ my @a = mce_map { $_ * $_ } 1..10000;
+ my @b = mce_map { $_ * $_ } \@list;
 
-   ## File_path, glob_ref, or scalar_ref
-   my @c = mce_map_f { chomp; $_ } "/path/to/file";
-   my @d = mce_map_f { chomp; $_ } $file_handle;
-   my @e = mce_map_f { chomp; $_ } \$scalar;
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_map { $_->[1] *= 2; $_ } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_map { $_->[1] *= 2; $_ } \@deeply_list;
 
-   ## Sequence of numbers (begin, end [, step, format])
-   my @f = mce_map_s { $_ * $_ } 1, 10000, 5;
-   my @g = mce_map_s { $_ * $_ } [ 1, 10000, 5 ];
+ ## File path, glob ref, IO::All::{ File, Pipe, STDIO } obj, or scalar ref
+ ## Workers read directly and not involve the manager process
+ my @e = mce_map_f { chomp; $_ } "/path/to/file"; # efficient
 
-   my @h = mce_map_s { $_ * $_ } {
-      begin => 1, end => 10000, step => 5, format => undef
-   };
+ ## Involves the manager process, therefore slower
+ my @f = mce_map_f { chomp; $_ } $file_handle;
+ my @g = mce_map_f { chomp; $_ } $io;
+ my @h = mce_map_f { chomp; $_ } \$scalar;
+
+ ## Sequence of numbers (begin, end [, step, format])
+ my @i = mce_map_s { $_ * $_ } 1, 10000, 5;
+ my @j = mce_map_s { $_ * $_ } [ 1, 10000, 5 ];
+
+ my @k = mce_map_s { $_ * $_ } {
+    begin => 1, end => 10000, step => 5, format => undef
+ };
 
 =head1 DESCRIPTION
 
@@ -485,20 +473,20 @@ MCE incurs a small overhead due to passing of data. A fast code block will
 run faster natively. However, the overhead will likely diminish as the
 complexity increases for the code.
 
-   my @m1 =     map { $_ * $_ } 1..1000000;               ## 0.127 secs
-   my @m2 = mce_map { $_ * $_ } 1..1000000;               ## 0.304 secs
+ my @m1 =     map { $_ * $_ } 1..1000000;               ## 0.127 secs
+ my @m2 = mce_map { $_ * $_ } 1..1000000;               ## 0.304 secs
 
 Chunking, enabled by default, greatly reduces the overhead behind the scene.
 The time for mce_map below also includes the time for data exchanges between
 the manager and worker processes. More parallelization will be seen when the
 code incurs additional CPU time.
 
-   sub calc {
-      sqrt $_ * sqrt $_ / 1.3 * 1.5 / 3.2 * 1.07
-   }
+ sub calc {
+    sqrt $_ * sqrt $_ / 1.3 * 1.5 / 3.2 * 1.07
+ }
 
-   my @m1 =     map { calc } 1..1000000;                  ## 0.367 secs
-   my @m2 = mce_map { calc } 1..1000000;                  ## 0.365 secs
+ my @m1 =     map { calc } 1..1000000;                  ## 0.367 secs
+ my @m2 = mce_map { calc } 1..1000000;                  ## 0.365 secs
 
 Even faster is mce_map_s; useful when input data is a range of numbers.
 Workers generate sequences mathematically among themselves without any
@@ -506,52 +494,49 @@ interaction from the manager process. Two arguments are required for
 mce_map_s (begin, end). Step defaults to 1 if begin is smaller than end,
 otherwise -1.
 
-   my @m3 = mce_map_s { calc } 1, 1000000;                ## 0.270 secs
+ my @m3 = mce_map_s { calc } 1, 1000000;                ## 0.270 secs
 
 Although this document is about MCE::Map, the L<MCE::Stream> module can write
 results immediately without waiting for all chunks to complete. This is made
 possible by passing the reference to an array (in this case @m4 and @m5).
 
-   use MCE::Stream;
+ use MCE::Stream;
 
-   sub calc {
-      sqrt $_ * sqrt $_ / 1.3 * 1.5 / 3.2 * 1.07
-   }
+ sub calc {
+    sqrt $_ * sqrt $_ / 1.3 * 1.5 / 3.2 * 1.07
+ }
 
-   my @m4; mce_stream \@m4, sub { calc }, 1..1000000;
+ my @m4; mce_stream \@m4, sub { calc }, 1..1000000;
 
-      ## Completes in 0.272 secs. This is amazing considering the
-      ## overhead for passing data between the manager and workers.
+    ## Completes in 0.272 secs. This is amazing considering the
+    ## overhead for passing data between the manager and workers.
 
-   my @m5; mce_stream_s \@m5, sub { calc }, 1, 1000000;
+ my @m5; mce_stream_s \@m5, sub { calc }, 1, 1000000;
 
-      ## Completed in 0.176 secs. Like with mce_map_s, specifying a
-      ## sequence specification turns out to be faster due to lesser
-      ## overhead for the manager process.
+    ## Completed in 0.176 secs. Like with mce_map_s, specifying a
+    ## sequence specification turns out to be faster due to lesser
+    ## overhead for the manager process.
 
 =head1 OVERRIDING DEFAULTS
 
 The following list options which may be overridden when loading the module.
 
-   use Sereal qw( encode_sereal decode_sereal );
-   use CBOR::XS qw( encode_cbor decode_cbor );
-   use JSON::XS qw( encode_json decode_json );
+ use Sereal qw( encode_sereal decode_sereal );
+ use CBOR::XS qw( encode_cbor decode_cbor );
+ use JSON::XS qw( encode_json decode_json );
 
-   use MCE::Map
-       max_workers => 4,                # Default 'auto'
-       chunk_size => 100,               # Default 'auto'
-       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
-       freeze => \&encode_sereal,       # \&Storable::freeze
-       thaw => \&decode_sereal          # \&Storable::thaw
-   ;
+ use MCE::Map
+     max_workers => 4,                # Default 'auto'
+     chunk_size => 100,               # Default 'auto'
+     tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+     freeze => \&encode_sereal,       # \&Storable::freeze
+     thaw => \&decode_sereal          # \&Storable::thaw
+ ;
 
-There is a simpler way to enable Sereal. The following will attempt to use
-Sereal if available, otherwise defaults to Storable for serialization.
+From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
+Specify C<< Sereal => 0 >> to use Storable instead.
 
-   use MCE::Map Sereal => 1;
-
-From MCE 1.800 onwards, this is done automatically if Sereal 3.008 or later
-is installed. Specify Sereal => 0 if Storable is desired.
+ use MCE::Map Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -561,50 +546,112 @@ is installed. Specify Sereal => 0 if Storable is desired.
 
 =item MCE::Map::init { options }
 
+=back
+
 The init function accepts a hash of MCE options. The gather option, if
 specified, is ignored due to being used internally by the module.
 
-   use MCE::Map;
+ use MCE::Map;
 
-   MCE::Map::init {
-      chunk_size => 1, max_workers => 4,
+ MCE::Map::init {
+    chunk_size => 1, max_workers => 4,
 
-      user_begin => sub {
-         print "## ", MCE->wid, " started\n";
-      },
+    user_begin => sub {
+       print "## ", MCE->wid, " started\n";
+    },
 
-      user_end => sub {
-         print "## ", MCE->wid, " completed\n";
-      }
-   };
+    user_end => sub {
+       print "## ", MCE->wid, " completed\n";
+    }
+ };
 
-   my @a = mce_map { $_ * $_ } 1..100;
+ my @a = mce_map { $_ * $_ } 1..100;
 
-   print "\n", "@a", "\n";
+ print "\n", "@a", "\n";
 
-   -- Output
+ -- Output
 
-   ## 2 started
-   ## 1 started
-   ## 3 started
-   ## 4 started
-   ## 1 completed
-   ## 4 completed
-   ## 2 completed
-   ## 3 completed
+ ## 2 started
+ ## 1 started
+ ## 3 started
+ ## 4 started
+ ## 1 completed
+ ## 4 completed
+ ## 2 completed
+ ## 3 completed
 
-   1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
-   400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
-   1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
-   2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
-   3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
-   5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
-   7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
-   10000
+ 1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
+ 400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
+ 1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
+ 2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
+ 3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
+ 5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
+ 7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
+ 10000
+
+=head1 API DOCUMENTATION
+
+=over 3
+
+=item MCE::Map->run ( sub { code }, list )
+
+=item mce_map { code } list
 
 =back
 
-=head1 API DOCUMENTATION
+Input data may be defined using a list or an array reference. Unlike MCE::Loop,
+Flow, and Step, specifying a hash reference as input data isn't allowed.
+
+ ## Array or array_ref
+ my @a = mce_map { $_ * 2 } 1..1000;
+ my @b = mce_map { $_ * 2 } \@list;
+
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_map { $_->[1] *= 2; $_ } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_map { $_->[1] *= 2; $_ } \@deeply_list;
+
+ ## Not supported
+ my @z = mce_map { ... } \%hash;
+
+=over 3
+
+=item MCE::Map->run_file ( sub { code }, file )
+
+=item mce_map_f { code } file
+
+=back
+
+The fastest of these is the /path/to/file. Workers communicate the next offset
+position among themselves with zero interaction by the manager process.
+
+C<IO::All> { File, Pipe, STDIO } is supported since MCE 1.845.
+
+ my @c = mce_map_f { chomp; $_ . "\r\n" } "/path/to/file";  # faster
+ my @d = mce_map_f { chomp; $_ . "\r\n" } $file_handle;
+ my @e = mce_map_f { chomp; $_ . "\r\n" } $io;              # IO::All
+ my @f = mce_map_f { chomp; $_ . "\r\n" } \$scalar;
+
+=over 3
+
+=item MCE::Map->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_map_s { code } $beg, $end [, $step, $fmt ]
+
+=back
+
+Sequence may be defined as a list, an array reference, or a hash reference.
+The functions require both begin and end values to run. Step and format are
+optional. The format is passed to sprintf (% may be omitted below).
+
+ my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
+
+ my @f = mce_map_s { $_ } $beg, $end, $step, $fmt;
+ my @g = mce_map_s { $_ } [ $beg, $end, $step, $fmt ];
+
+ my @h = mce_map_s { $_ } {
+    begin => $beg, end => $end,
+    step => $step, format => $fmt
+ };
 
 =over 3
 
@@ -612,50 +659,13 @@ specified, is ignored due to being used internally by the module.
 
 =item mce_map { code } iterator
 
-An iterator reference can by specified for input_data. Iterators are described
-under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   my @a = mce_map { $_ * 2 } make_iterator(10, 30, 2);
-
-=item MCE::Map->run ( sub { code }, list )
-
-=item mce_map { code } list
-
-Input data can be defined using a list.
-
-   my @a = mce_map { $_ * 2 } 1..1000;
-   my @b = mce_map { $_ * 2 } [ 1..1000 ];
-
-=item MCE::Map->run_file ( sub { code }, file )
-
-=item mce_map_f { code } file
-
-The fastest of these is the /path/to/file. Workers communicate the next offset
-position among themselves without any interaction from the manager process.
-
-   my @c = mce_map_f { chomp; $_ . "\r\n" } "/path/to/file";
-   my @d = mce_map_f { chomp; $_ . "\r\n" } $file_handle;
-   my @e = mce_map_f { chomp; $_ . "\r\n" } \$scalar;
-
-=item MCE::Map->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
-
-=item mce_map_s { code } $beg, $end [, $step, $fmt ]
-
-Sequence can be defined as a list, an array reference, or a hash reference.
-The functions require both begin and end values to run. Step and format are
-optional. The format is passed to sprintf (% may be omitted below).
-
-   my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
-
-   my @f = mce_map_s { $_ } $beg, $end, $step, $fmt;
-   my @g = mce_map_s { $_ } [ $beg, $end, $step, $fmt ];
-
-   my @h = mce_map_s { $_ } {
-      begin => $beg, end => $end, step => $step, format => $fmt
-   };
-
 =back
 
+An iterator reference may be specified for input_data. Iterators are described
+under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
+
+ my @a = mce_map { $_ * 2 } make_iterator(10, 30, 2);
+
 =head1 MANUAL SHUTDOWN
 
 =over 3
@@ -664,21 +674,21 @@ optional. The format is passed to sprintf (% may be omitted below).
 
 =item MCE::Map::finish
 
+=back
+
 Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
 
-   use MCE::Map;
+ use MCE::Map;
 
-   MCE::Map::init {
-      chunk_size => 20, max_workers => 'auto'
-   };
+ MCE::Map::init {
+    chunk_size => 20, max_workers => 'auto'
+ };
 
-   my @a = mce_map { ... } 1..100;
+ my @a = mce_map { ... } 1..100;
 
-   MCE::Map::finish;
-
-=back
+ MCE::Map::finish;
 
 =head1 INDEX
 
