@@ -11,24 +11,23 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.799_03';
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use Scalar::Util qw( looks_like_number );
+
 use MCE;
 use MCE::Queue;
 
 our @CARP_NOT = qw( MCE );
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
-my $_caller;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ###############################################################################
@@ -48,7 +47,6 @@ sub import {
       MAX_WORKERS  => 'auto',
       CHUNK_SIZE   => 'auto',
       DEFAULT_MODE => 'map',
-      FAST         => 0,
    };
 
    ## Import functions.
@@ -68,9 +66,10 @@ sub import {
       $_p->{FREEZE}       = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}         = shift, next if ( $_arg eq 'thaw' );
       $_p->{DEFAULT_MODE} = shift, next if ( $_arg eq 'default_mode' );
-      $_p->{FAST}         = shift, next if ( $_arg eq 'fast' );
 
-      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+                            shift, next if ( $_arg eq 'fast' ); # ignored
+
+      ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
          if ( shift eq '0' ) {
             require Storable;
@@ -86,19 +85,15 @@ sub import {
    _croak("Error: (DEFAULT_MODE) is not valid")
       if ($_p->{DEFAULT_MODE} ne 'grep' && $_p->{DEFAULT_MODE} ne 'map');
 
-   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+   $_p->{MAX_WORKERS} = MCE::_parse_max_workers($_p->{MAX_WORKERS});
 
-   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
-   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+   MCE::_validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS', $_tag);
+   MCE::_validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE', $_tag)
       unless ($_p->{CHUNK_SIZE} eq 'auto');
 
    return;
 }
 
-END {
-   %{ $_MCE } = ();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Gather callback to ensure chunk order is preserved during gathering.
@@ -152,10 +147,13 @@ sub _task_end {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
-   MCE::Stream->finish( my $_pkg = "$$.$_tid.".caller() );
+   my $_pkg = "$$.$_tid.".caller();
 
    $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
+   _croak("$_tag: (HASH) not allowed as input by this MCE model")
+      if ( ref $_params->{$_pkg}{input_data} eq 'HASH' );
+
    @_ = ();
 
    return;
@@ -166,14 +164,11 @@ sub finish (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
    my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
 
-   if ( $_pkg eq 'MCE::Shared::Server' ) {
-      MCE::Stream->finish($_) for ( keys %{ $_MCE } );
-      %{ $_MCE } = ();
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Stream->finish($_k, 1); }
    }
-   elsif ( exists $_MCE->{$_pkg} ) {
-      MCE::_save_state(), $_MCE->{$_pkg}->shutdown(@_), MCE::_restore_state()
-         if $_MCE->{$_pkg}{_spawned};
-
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
       $_gather_ref = $_order_id = undef, undef %_tmp;
 
       delete $_user_tasks->{$_pkg};
@@ -181,6 +176,7 @@ sub finish (@) {
       delete $_prev_m->{$_pkg};
       delete $_prev_n->{$_pkg};
       delete $_prev_w->{$_pkg};
+      delete $_MCE->{$_pkg};
 
       if (defined $_queue->{$_pkg}) {
          local $_;
@@ -189,6 +185,8 @@ sub finish (@) {
       }
    }
 
+   @_ = ();
+
    return;
 }
 
@@ -203,7 +201,7 @@ sub run_file (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_file, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
-   my $_pid = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{input_data} if (exists $_p->{input_data});
@@ -215,7 +213,7 @@ sub run_file (@) {
 
    for my $_i ($_start_pos .. @_ - 1) {
       my $_r = ref $_[$_i];
-      if ($_r eq '' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+      if ($_r eq '' || $_r eq 'SCALAR' || $_r =~ /^(?:GLOB|FileHandle|IO::)/) {
          $_file = $_[$_i]; $_pos = $_i;
          last;
       }
@@ -227,7 +225,7 @@ sub run_file (@) {
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
       $_params->{$_pid}{_file} = $_file;
    }
    else {
@@ -252,7 +250,7 @@ sub run_seq (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_begin, $_end, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
-   my $_pid = "$$.$_tid.".caller();  $_caller = caller();
+   my $_pid = "$$.$_tid.".caller();
 
    if (defined (my $_p = $_params->{$_pid})) {
       delete $_p->{sequence}   if (exists $_p->{sequence});
@@ -264,22 +262,22 @@ sub run_seq (@) {
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
-      my $_ref = ref $_[$_i];
+      my $_r = ref $_[$_i];
 
-      if ($_ref eq '' || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
+      if ($_r eq '' || $_r =~ /^Math::/ || $_r eq 'HASH' || $_r eq 'ARRAY') {
          $_pos = $_i;
 
-         if ($_ref eq '') {
+         if ($_r eq '' || $_r =~ /^Math::/) {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
             $_params->{$_pid}{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
             ];
          }
-         elsif ($_ref eq 'HASH') {
+         elsif ($_r eq 'HASH') {
             $_begin = $_[$_pos]->{begin}; $_end = $_[$_pos]->{end};
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
-         elsif ($_ref eq 'ARRAY') {
+         elsif ($_r eq 'ARRAY') {
             $_begin = $_[$_pos]->[0]; $_end = $_[$_pos]->[1];
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
@@ -295,7 +293,7 @@ sub run_seq (@) {
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{$_pid}{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = undef;
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
@@ -314,7 +312,7 @@ sub run (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
-   my $_pkg = defined $_caller ? $_caller : caller();  $_caller = undef;
+   my $_pkg = caller() eq 'MCE::Stream' ? caller(1) : caller();
    my $_pid = "$$.$_tid.$_pkg";
 
    if (ref $_[0] eq 'HASH' && !exists $_[0]->{code}) {
@@ -347,6 +345,8 @@ sub run (@) {
          push @_mode, $_default_mode;
       }
       else {
+         last if (!exists $_[0]->{code} && !exists $_[0]->{mode});
+
          push @_code, exists $_[0]->{code} ? $_[0]->{code} : undef;
          push @_mode, exists $_[0]->{mode} ? $_[0]->{mode} : $_default_mode;
 
@@ -401,12 +401,14 @@ sub run (@) {
    my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
    my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+   if (@_ == 1 && $_r =~ /^(?:ARRAY|HASH|SCALAR|GLOB|FileHandle|IO::)/) {
+      _croak("$_tag: (HASH) not allowed as input by this MCE model")
+         if $_r eq 'HASH';
       $_input_data = shift;
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
-      $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
+      $_max_workers = MCE::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers} && ref $_p->{max_workers} ne 'ARRAY');
 
       delete $_p->{sequence}    if (defined $_input_data || scalar @_);
@@ -421,7 +423,7 @@ sub run (@) {
       $_max_workers = int($_max_workers / @_code + 0.5) + 1;
    }
 
-   my $_chunk_size = MCE::Util::_parse_chunk_size(
+   my $_chunk_size = MCE::_parse_chunk_size(
       $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
       $_input_data, scalar @_
    );
@@ -436,19 +438,22 @@ sub run (@) {
 
    ## -------------------------------------------------------------------------
 
-   MCE::_save_state();
+   MCE::_save_state($_MCE->{$_pid});
 
-   if ($_init_mce) {
+   if ($_init_mce || !exists $_queue->{$_pid}) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
       $_queue->{$_pid} = [] if (!defined $_queue->{$_pid});
 
       my $_Q = $_queue->{$_pid};
       pop(@{ $_Q })->DESTROY for (@_code .. @{ $_Q });
 
-      push @{ $_Q }, MCE::Queue->new(fast => $_def->{$_pkg}{FAST})
+      push @{ $_Q }, MCE::Queue->new()
          for (@{ $_Q } .. @_code - 2);
 
-      _gen_user_tasks($_pid, $_Q, \@_code, \@_mode, \@_name, \@_wrks);
+      ## must clear arrays for nested session to work with Perl < v5.14
+      _gen_user_tasks($_pid, $_Q, [@_code], [@_mode], [@_name], [@_wrks]);
+
+      @_code = @_mode = @_name = @_wrks = ();
 
       my %_opts = (
          max_workers => $_max_workers, task_name => $_tag,
@@ -467,7 +472,7 @@ sub run (@) {
             next if ($_ eq 'chunk_size');
             next if ($_ eq 'task_end');
 
-            _croak("MCE::Stream: ($_) is not a valid constructor argument")
+            _croak("$_tag: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
             $_opts{$_} = $_p->{$_};
@@ -520,14 +525,14 @@ sub run (@) {
       }
    }
 
-   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
-      $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
-      $_->DESTROY() for (@{ $_queue->{$_pid} });
+   MCE::_restore_state();
+
+   # destroy queue(s) if MCE::run requested workers to shutdown
+   if (!$_MCE->{$_pid}{_spawned}) {
+      $_->DESTROY() for @{ $_queue->{$_pid} };
       delete $_queue->{$_pid};
    }
 
-   MCE::_restore_state();
-
    return map { @{ $_ } } delete @_tmp{ 1 .. $_order_id - 1 }
       unless (defined $_aref);
 
@@ -650,21 +655,6 @@ sub _gen_user_tasks {
    return;
 }
 
-sub _validate_number {
-
-   my ($_n, $_key) = @_;
-
-   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
-
-   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
-
-   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
-      _croak("$_tag: ($_key) is not valid");
-   }
-
-   return;
-}
-
 1;
 
 __END__
@@ -681,43 +671,51 @@ MCE::Stream - Parallel stream model for chaining multiple maps and greps
 
 =head1 VERSION
 
-This document describes MCE::Stream version 1.799_03
+This document describes MCE::Stream version 1.867_01
 
 =head1 SYNOPSIS
 
-   ## Exports mce_stream, mce_stream_f, mce_stream_s
-   use MCE::Stream;
+ ## Exports mce_stream, mce_stream_f, mce_stream_s
+ use MCE::Stream;
 
-   my (@m1, @m2, @m3);
+ my (@m1, @m2, @m3);
 
-   ## Default mode is map and processed from right-to-left
-   @m1 = mce_stream sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
-   mce_stream \@m2, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
+ ## Default mode is map and processed from right-to-left
+ @m1 = mce_stream sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
+ mce_stream \@m2, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
 
-   ## Native Perl
-   @m3 = map { $_ * $_ } grep { $_ % 5 == 0 } 1..10000;
+ ## Native Perl
+ @m3 = map { $_ * $_ } grep { $_ % 5 == 0 } 1..10000;
 
-   ## Streaming grep and map in parallel
-   mce_stream \@m3,
-      { mode => 'map',  code => sub { $_ * $_ } },
-      { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
+ ## Streaming grep and map in parallel
+ mce_stream \@m3,
+    { mode => 'map',  code => sub { $_ * $_ } },
+    { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
 
-   ## Array or array_ref
-   my @a = mce_stream sub { $_ * $_ }, 1..10000;
-   my @b = mce_stream sub { $_ * $_ }, [ 1..10000 ];
+ ## Array or array_ref
+ my @a = mce_stream sub { $_ * $_ }, 1..10000;
+ my @b = mce_stream sub { $_ * $_ }, \@list;
 
-   ## File_path, glob_ref, or scalar_ref
-   my @c = mce_stream_f sub { chomp; $_ }, "/path/to/file";
-   my @d = mce_stream_f sub { chomp; $_ }, $file_handle;
-   my @e = mce_stream_f sub { chomp; $_ }, \$scalar;
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_stream sub { $_->[1] *= 2; $_ }, [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_stream sub { $_->[1] *= 2; $_ }, \@deeply_list;
 
-   ## Sequence of numbers (begin, end [, step, format])
-   my @f = mce_stream_s sub { $_ * $_ }, 1, 10000, 5;
-   my @g = mce_stream_s sub { $_ * $_ }, [ 1, 10000, 5 ];
+ ## File path, glob ref, IO::All::{ File, Pipe, STDIO } obj, or scalar ref
+ ## Workers read directly and not involve the manager process
+ my @e = mce_stream_f sub { chomp; $_ }, "/path/to/file"; # efficient
 
-   my @h = mce_stream_s sub { $_ * $_ }, {
-      begin => 1, end => 10000, step => 5, format => undef
-   };
+ ## Involves the manager process, therefore slower
+ my @f = mce_stream_f sub { chomp; $_ }, $file_handle;
+ my @g = mce_stream_f sub { chomp; $_ }, $io;
+ my @h = mce_stream_f sub { chomp; $_ }, \$scalar;
+
+ ## Sequence of numbers (begin, end [, step, format])
+ my @i = mce_stream_s sub { $_ * $_ }, 1, 10000, 5;
+ my @j = mce_stream_s sub { $_ * $_ }, [ 1, 10000, 5 ];
+
+ my @k = mce_stream_s sub { $_ * $_ }, {
+    begin => 1, end => 10000, step => 5, format => undef
+ };
 
 =head1 DESCRIPTION
 
@@ -725,39 +723,39 @@ This module allows one to stream multiple map and/or grep operations in
 parallel. Code blocks run simultaneously from right-to-left. The results
 are appended immediately when providing a reference to an array.
 
-   ## Appends are serialized, even out-of-order ok, but immediately.
-   ## Out-of-order chunks are held temporarily until ordered chunks
-   ## arrive.
+ ## Appends are serialized, even out-of-order ok, but immediately.
+ ## Out-of-order chunks are held temporarily until ordered chunks
+ ## arrive.
 
-   mce_stream \@a, sub { $_ }, sub { $_ }, sub { $_ }, 1..10000;
+ mce_stream \@a, sub { $_ }, sub { $_ }, sub { $_ }, 1..10000;
 
-   ##                                                    input
-   ##                                        chunk1      input
-   ##                            chunk3      chunk2      input
-   ##                chunk2      chunk2      chunk3      input
-   ##   append1      chunk3      chunk1      chunk4      input
-   ##   append2      chunk1      chunk5      chunk5      input
-   ##   append3      chunk5      chunk4      chunk6      ...
-   ##   append4      chunk4      chunk6      ...
-   ##   append5      chunk6      ...
-   ##   append6      ...
-   ##   ...
-   ##
+ ##                                                    input
+ ##                                        chunk1      input
+ ##                            chunk3      chunk2      input
+ ##                chunk2      chunk2      chunk3      input
+ ##   append1      chunk3      chunk1      chunk4      input
+ ##   append2      chunk1      chunk5      chunk5      input
+ ##   append3      chunk5      chunk4      chunk6      ...
+ ##   append4      chunk4      chunk6      ...
+ ##   append5      chunk6      ...
+ ##   append6      ...
+ ##   ...
+ ##
 
 MCE incurs a small overhead due to passing of data. A fast code block will
 run faster natively when chaining multiple map functions. However, the
 overhead will likely diminish as the complexity increases for the code.
 
-   ## 0.334 secs -- baseline using the native map function
-   my @m1 = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..1000000;
+ ## 0.334 secs -- baseline using the native map function
+ my @m1 = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..1000000;
 
-   ## 0.427 secs -- this is quite amazing considering data passing
-   my @m2 = mce_stream
-         sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
+ ## 0.427 secs -- this is quite amazing considering data passing
+ my @m2 = mce_stream
+       sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
 
-   ## 0.355 secs -- appends to @m3 immediately, not after running
-   my @m3; mce_stream \@m3,
-         sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
+ ## 0.355 secs -- appends to @m3 immediately, not after running
+ my @m3; mce_stream \@m3,
+       sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
 
 Even faster is mce_stream_s; useful when input data is a range of numbers.
 Workers generate sequences mathematically among themselves without any
@@ -765,35 +763,33 @@ interaction from the manager process. Two arguments are required for
 mce_stream_s (begin, end). Step defaults to 1 if begin is smaller than end,
 otherwise -1.
 
-   ## 0.278 secs -- numbers are generated mathematically via sequence
-   my @m4; mce_stream_s \@m4,
-         sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1, 1000000;
+ ## 0.278 secs -- numbers are generated mathematically via sequence
+ my @m4; mce_stream_s \@m4,
+       sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1, 1000000;
 
 =head1 OVERRIDING DEFAULTS
 
 The following list options which may be overridden when loading the module.
+The fast option is obsolete in 1.867 onwards; ignored if specified.
 
-   use Sereal qw( encode_sereal decode_sereal );
-   use CBOR::XS qw( encode_cbor decode_cbor );
-   use JSON::XS qw( encode_json decode_json );
+ use Sereal qw( encode_sereal decode_sereal );
+ use CBOR::XS qw( encode_cbor decode_cbor );
+ use JSON::XS qw( encode_json decode_json );
 
-   use MCE::Stream
-       max_workers => 8,                # Default 'auto'
-       chunk_size => 500,               # Default 'auto'
-       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
-       freeze => \&encode_sereal,       # \&Storable::freeze
-       thaw => \&decode_sereal,         # \&Storable::thaw
-       default_mode => 'grep',          # Default 'map'
-       fast => 1                        # Default 0 (fast dequeue)
-   ;
+ use MCE::Stream
+     max_workers => 8,                # Default 'auto'
+     chunk_size => 500,               # Default 'auto'
+     tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+     freeze => \&encode_sereal,       # \&Storable::freeze
+     thaw => \&decode_sereal,         # \&Storable::thaw
+     default_mode => 'grep',          # Default 'map'
+     fast => 1                        # Default 0 (fast dequeue)
+ ;
 
-There is a simpler way to enable Sereal. The following will attempt to use
-Sereal if available, otherwise defaults to Storable for serialization.
+From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
+Specify C<< Sereal => 0 >> to use Storable instead.
 
-   use MCE::Stream Sereal => 1;
-
-From MCE 1.800 onwards, this is done automatically if Sereal 3.008 or later
-is installed. Specify Sereal => 0 if Storable is desired.
+ use MCE::Stream Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -803,49 +799,49 @@ is installed. Specify Sereal => 0 if Storable is desired.
 
 =item MCE::Stream::init { options }
 
+=back
+
 The init function accepts a hash of MCE options. The gather and bounds_only
 options, if specified, are ignored due to being used internally by the
 module (not shown below).
 
-   use MCE::Stream;
+ use MCE::Stream;
 
-   MCE::Stream::init {
-      chunk_size => 1, max_workers => 4,
+ MCE::Stream::init {
+    chunk_size => 1, max_workers => 4,
 
-      user_begin => sub {
-         print "## ", MCE->wid, " started\n";
-      },
+    user_begin => sub {
+       print "## ", MCE->wid, " started\n";
+    },
 
-      user_end => sub {
-         print "## ", MCE->wid, " completed\n";
-      }
-   };
+    user_end => sub {
+       print "## ", MCE->wid, " completed\n";
+    }
+ };
 
-   my @a = mce_stream sub { $_ * $_ }, 1..100;
+ my @a = mce_stream sub { $_ * $_ }, 1..100;
 
-   print "\n", "@a", "\n";
+ print "\n", "@a", "\n";
 
-   -- Output
+ -- Output
 
-   ## 1 started
-   ## 2 started
-   ## 3 started
-   ## 4 started
-   ## 3 completed
-   ## 1 completed
-   ## 2 completed
-   ## 4 completed
+ ## 1 started
+ ## 2 started
+ ## 3 started
+ ## 4 started
+ ## 3 completed
+ ## 1 completed
+ ## 2 completed
+ ## 4 completed
 
-   1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
-   400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
-   1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
-   2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
-   3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
-   5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
-   7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
-   10000
-
-=back
+ 1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
+ 400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
+ 1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
+ 2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
+ 3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
+ 5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
+ 7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
+ 10000
 
 Like with MCE::Stream::init above, MCE options may be specified using an
 anonymous hash for the first argument. Notice how both max_workers and
@@ -855,47 +851,47 @@ per each code block.
 Remember that MCE::Stream processes from right-to-left when setting the
 individual values.
 
-   use MCE::Stream;
+ use MCE::Stream;
 
-   my @a = mce_stream {
-      task_name   => [ 'c', 'b', 'a' ],
-      max_workers => [  2,   4,   3, ],
+ my @a = mce_stream {
+    task_name   => [ 'c', 'b', 'a' ],
+    max_workers => [  2,   4,   3, ],
 
-      user_end => sub {
-         my ($mce, $task_id, $task_name) = @_;
-         print "$task_id - $task_name completed\n";
-      },
+    user_end => sub {
+       my ($mce, $task_id, $task_name) = @_;
+       print "$task_id - $task_name completed\n";
+    },
 
-      task_end => sub {
-         my ($mce, $task_id, $task_name) = @_;
-         MCE->print("$task_id - $task_name ended\n");
-      }
-   },
-   sub { $_ * 4 },             ## 2 workers, named c
-   sub { $_ * 3 },             ## 4 workers, named b
-   sub { $_ * 2 }, 1..10000;   ## 3 workers, named a
+    task_end => sub {
+       my ($mce, $task_id, $task_name) = @_;
+       MCE->print("$task_id - $task_name ended\n");
+    }
+ },
+ sub { $_ * 4 },             ## 2 workers, named c
+ sub { $_ * 3 },             ## 4 workers, named b
+ sub { $_ * 2 }, 1..10000;   ## 3 workers, named a
 
-   -- Output
+ -- Output
 
-   0 - a completed
-   0 - a completed
-   0 - a completed
-   0 - a ended
-   1 - b completed
-   1 - b completed
-   1 - b completed
-   1 - b completed
-   1 - b ended
-   2 - c completed
-   2 - c completed
-   2 - c ended
+ 0 - a completed
+ 0 - a completed
+ 0 - a completed
+ 0 - a ended
+ 1 - b completed
+ 1 - b completed
+ 1 - b completed
+ 1 - b completed
+ 1 - b ended
+ 2 - c completed
+ 2 - c completed
+ 2 - c ended
 
 Note that the anonymous hash, for specifying options, also comes first when
 passing an array reference.
 
-   my @a; mce_stream {
-      ...
-   }, \@a, sub { ... }, sub { ... }, 1..10000;
+ my @a; mce_stream {
+    ...
+ }, \@a, sub { ... }, sub { ... }, 1..10000;
 
 =head1 API DOCUMENTATION
 
@@ -903,24 +899,85 @@ Scripts using MCE::Stream can be written using the long or short form.
 The long form becomes relevant when mixing modes. Again, processing
 occurs from right-to-left.
 
-   my @m3 = mce_stream
-      { mode => 'map',  code => sub { $_ * $_ } },
-      { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
+ my @m3 = mce_stream
+    { mode => 'map',  code => sub { $_ * $_ } },
+    { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
 
-   my @m4; mce_stream \@m4,
-      { mode => 'map',  code => sub { $_ * $_ } },
-      { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
+ my @m4; mce_stream \@m4,
+    { mode => 'map',  code => sub { $_ * $_ } },
+    { mode => 'grep', code => sub { $_ % 5 == 0 } }, 1..10000;
 
 For multiple grep blocks, the short form can be used. Simply specify the
 default mode for the module. The two valid values for default_mode is 'grep'
 and 'map'.
 
-   use MCE::Stream default_mode => 'grep';
+ use MCE::Stream default_mode => 'grep';
 
-   my @f = mce_stream_f sub { /ending$/ }, sub { /^starting/ }, $file;
+ my @f = mce_stream_f sub { /ending$/ }, sub { /^starting/ }, $file;
 
 The following assumes 'map' for default_mode in order to demonstrate all the
-possibilities of passing input data into the code block.
+possibilities for providing input data.
+
+=over 3
+
+=item MCE::Stream->run ( sub { code }, list )
+
+=item mce_stream sub { code }, list
+
+=back
+
+Input data may be defined using a list or an array reference. Unlike MCE::Loop,
+Flow, and Step, specifying a hash reference as input data isn't allowed.
+
+ ## Array or array_ref
+ my @a = mce_stream sub { $_ * 2 }, 1..1000;
+ my @b = mce_stream sub { $_ * 2 }, \@list;
+
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_stream sub { $_->[1] *= 2; $_ }, [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_stream sub { $_->[1] *= 2; $_ }, \@deeply_list;
+
+ ## Not supported
+ my @z = mce_stream sub { ... }, \%hash;
+
+=over 3
+
+=item MCE::Stream->run_file ( sub { code }, file )
+
+=item mce_stream_f sub { code }, file
+
+=back
+
+The fastest of these is the /path/to/file. Workers communicate the next offset
+position among themselves with zero interaction by the manager process.
+
+C<IO::All> { File, Pipe, STDIO } is supported since MCE 1.845.
+
+ my @c = mce_stream_f sub { chomp; $_ . "\r\n" }, "/path/to/file";  # faster
+ my @d = mce_stream_f sub { chomp; $_ . "\r\n" }, $file_handle;
+ my @e = mce_stream_f sub { chomp; $_ . "\r\n" }, $io;              # IO::All
+ my @f = mce_stream_f sub { chomp; $_ . "\r\n" }, \$scalar;
+
+=over 3
+
+=item MCE::Stream->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_stream_s sub { code }, $beg, $end [, $step, $fmt ]
+
+=back
+
+Sequence may be defined as a list, an array reference, or a hash reference.
+The functions require both begin and end values to run. Step and format are
+optional. The format is passed to sprintf (% may be omitted below).
+
+ my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
+
+ my @f = mce_stream_s sub { $_ }, $beg, $end, $step, $fmt;
+ my @g = mce_stream_s sub { $_ }, [ $beg, $end, $step, $fmt ];
+
+ my @h = mce_stream_s sub { $_ }, {
+    begin => $beg, end => $end, step => $step, format => $fmt
+ };
 
 =over 3
 
@@ -928,57 +985,20 @@ possibilities of passing input data into the code block.
 
 =item mce_stream { input_data => iterator }, sub { code }
 
-An iterator reference can by specified for input_data. The only other way
+=back
+
+An iterator reference may be specified for input_data. The only other way
 is to specify input_data via MCE::Stream::init. This prevents MCE::Stream
 from configuring the iterator reference as another user task which will
 not work.
 
-Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
+Iterators are described under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
 
-   MCE::Stream::init {
-      input_data => iterator
-   };
+ MCE::Stream::init {
+    input_data => iterator
+ };
 
-   my @a = mce_stream sub { $_ * 3 }, sub { $_ * 2 };
-
-=item MCE::Stream->run ( sub { code }, list )
-
-=item mce_stream sub { code }, list
-
-Input data can be defined using a list.
-
-   my @a = mce_stream sub { $_ * 2 }, 1..1000;
-   my @b = mce_stream sub { $_ * 2 }, [ 1..1000 ];
-
-=item MCE::Stream->run_file ( sub { code }, file )
-
-=item mce_stream_f sub { code }, file
-
-The fastest of these is the /path/to/file. Workers communicate the next offset
-position among themselves without any interaction from the manager process.
-
-   my @c = mce_stream_f sub { chomp; $_ . "\r\n" }, "/path/to/file";
-   my @d = mce_stream_f sub { chomp; $_ . "\r\n" }, $file_handle;
-   my @e = mce_stream_f sub { chomp; $_ . "\r\n" }, \$scalar;
-
-=item MCE::Stream->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
-
-=item mce_stream_s sub { code }, $beg, $end [, $step, $fmt ]
-
-Sequence can be defined as a list, an array reference, or a hash reference.
-The functions require both begin and end values to run. Step and format are
-optional. The format is passed to sprintf (% may be omitted below).
-
-   my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
-
-   my @f = mce_stream_s sub { $_ }, $beg, $end, $step, $fmt;
-   my @g = mce_stream_s sub { $_ }, [ $beg, $end, $step, $fmt ];
-
-   my @h = mce_stream_s sub { $_ }, {
-      begin => $beg, end => $end, step => $step, format => $fmt
-   };
-
-=back
+ my @a = mce_stream sub { $_ * 3 }, sub { $_ * 2 };
 
 =head1 MANUAL SHUTDOWN
 
@@ -988,21 +1008,21 @@ optional. The format is passed to sprintf (% may be omitted below).
 
 =item MCE::Stream::finish
 
+=back
+
 Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
 
-   use MCE::Stream;
+ use MCE::Stream;
 
-   MCE::Stream::init {
-      chunk_size => 20, max_workers => 'auto'
-   };
+ MCE::Stream::init {
+    chunk_size => 20, max_workers => 'auto'
+ };
 
-   my @a = mce_stream { ... } 1..100;
+ my @a = mce_stream { ... } 1..100;
 
-   MCE::Stream::finish;
-
-=back
+ MCE::Stream::finish;
 
 =head1 INDEX
 
